c   main program name   - ellipse
c 
c-----------------------------------------------------------------------
c   
c   Computer            - SUN-Sparc satation 2
c
c   Latest revision     - Jan.19, 1996
c
c   Purpose             - Extracts the contours of cells/fibres from a 
c                           black and white image file and generates a 
c                           PostScript file that shows extracted cell
c                           contours and lists their barycenters.
c                          
c   Required routines   - formatPS, headerPS.dat, extgf, incell, 
c                         bound, center, wPS
c
c   Written by          - Miroslav Dujin
c   Modified by         - Mingshan Li
c   Modified by         - Dakshina Murthy (changed the algo to be fit for 
c                                      scanning any type of particle.
c   
c   Instruction         - This programm was modified from Miroslav 
c                         Dujin's program:genfib.f which used boundary
c                         fitting points to extract simulation ellipses.
c                         This program obtained the simulation ellipses
c                         by comparing the actual particles' (in pixel
c                         level) and the simulation ellipses' 0-order,
c                         1stXS-order and 2nd-order moments (I0(A), 
c                         I1x, I1y, Ixx and Iyy).  The fitting
c                         points method was used to make the first guess 
c                         for the particle's orientation.
c
c-----------------------------------------------------------------------
c     
      program           ellipse_from_particles
c     
      implicit none
      integer           k,kp,maxr,maxc
      parameter         (k=1000)     ! max. expected no. of cells in image
      parameter         (kp = 50)   ! max. no. of pts. on contour
      parameter         (maxr=1000,maxc=1000) ! image domain
c                                  Specifications for variables
      integer           unit,step,sep,cols,rows,img(maxr,maxc),j,l,
     *                  bits,recl,rec0,nc,i,rc(k),cc(k),npol
      real              pi,fl,fw,fx,fy,step0,sep0,dpi,length,exc,
     *                  width,a(k),b(k),th(k),area(k),xypolyv(1000,2),
     *                  tim,dtime,tarr(2),thf,hr,mn,sc,sf1,sf2,psize,
     *                  apixel,ximg(maxr,maxc),yimg(maxr,maxc),vref
      character*72      namei0,namei,nameo1,nameo2,nameps1,
     *                  nameps2,nameps3
      integer      narray(maxr,maxc),newarray(maxr,maxc),affo(maxr,maxc)
     *                  ,mnpart(k),nell,narrayini(1000000),polyflag(150)
      integer      npvert_total,netemp,npv(150),connect(1000,100)
      data              unit/101/,length/11.0/,width/8.5/
      common            pi
c                                  First executable statement
      pi = 4.*atan(1.)
c                                  Read block
      open(119,file='ell8.inp')
      read (119,*)namei0
      read (119,*)nameo1
      read (119,*) fw                     ! conv.to inches
      read (119,*) exc
      close(119)
c                                  CPU time initialization:
      tim = dtime(tarr)

      open(unit,file=namei0,status='old')
      read(unit,*)
      read(unit,*)
      read(unit,*) cols,rows

      dpi = cols/fw                ! pixels per inch (for one image line)
      fl = rows/dpi                ! image frame lenght in inches 
      psize=0.d0
      do i=1,int(rows*cols/35.0)
         read (unit,*) (narrayini((i-1)*35+j),j=1,35)
      enddo
      if(mod(rows*cols,35).gt.0) 
     * read (unit,*) (narrayini(int(rows*cols/35.0)*35+j),
     *                                   j=1,mod(rows*cols,35))
      close(unit)
      do i=1,rows
         do j=1,cols
            img(i,j)=narrayini(cols*(i-1)+j)
            if(img(i,j).eq.0) then
               img(i,j)=1
            else  
               img(i,j)=0
            endif
         enddo
      enddo                 

c ! in pixels or degree,origin in left top, output on screen
c
      do i=1,rows
         do j=1,cols
            narray(i,j)=img(i,j)
         enddo
      enddo
      vref=0
      call shape_characteristics(narray,affo,rows,cols,npol,polyflag,vref)
      call extractBM (img,ximg,yimg,a,b,rc,cc,th,area,nc,step,sep,exc,
     *                rows,cols,apixel,dpi,sf1,sf2,affo,mnpart,vref)
      close(unit)
c get polygon data
      
      if(npol.ne.0) then
         open(199,file='poly.out')
         read(199,*) npvert_total,netemp
         do i=1,npvert_total
            read(199,*) j,xypolyv(i,1),xypolyv(i,2)
         enddo
         do i=1,netemp
            if(polyflag(i).eq.0) read(199,*)
            if(polyflag(i).ne.0) read(199,*)j,npv(j),
     *                                  (connect(i,l),l=1,npv(j))
         enddo         
      endif      

      print *,'Number of extracted cell contours:', nc
      print *
      print *,' num  xc(inch)   yc(inch) '    
      print *,'-----------------------------'
      do 10 i=1,nc
         write (*,800) i,cc(i)/dpi,(rows-rc(i))/dpi   ! in inch
c ! in inch,origin in left bottom, output on screen
 10   continue
c
 800  format(i3,1x,4x,f7.3,4x,f7.3)
c

      step0=step0*25.4  ! conv.to milimeter
      sep0=sep0*25.4  ! conv.to milimeter

c write output data file (input in MESH genertion)
      call woutdat1 (nameo1,rows,cols,dpi,nc,a,b,cc,rc,th,fw,fl,psize,npv,
     *                      connect,xypolyv,polyflag)
      stop
      end

C =============================================================

c   routine name        - extractBM
c                       
c-----------------------------------------------------------------------
c   
c   Computer            - HP/apollo/715/33
c
c   Latest revision     - Jan.18, 1996
c
c   Purpose             - Extracts the ellipses from a black
c                           and white PostScript image file.
c     
c   Usage               - call extractBM (img,ximg,yimg,a,b,rc,cc,th,
c                                         area,ne,step,sep,exc,
c                                   rows,cols,apixel,dpi,sf,affo,mnpart)
c
c   Arguments  
c                 img   - 2D integer matrix representing image domain
c                           1-white pxl., 0-black.(input)
c                 a     - a real array of length ne containing 
c                           the major semi-axes of the ellipses. 
c                           (output)
c                 b     - a real array of length ne containing
c                           the minor semi-axes of the ellipses.
c                           (output)
c                 rc    - a real array of length ne containing the
c                           row coordinates of the centers of the
c                           ellipses. (output) 
c                 cc    - a real array of length ne containing the
c                           column coordinates of the centers of the
c                           ellipses. (output) 
c                 th    - a real array of length ne containing
c                           inclination angles of the ellipses.
c                           (output)
c                 ne    - number of ellipses. 
c                           (output)
c                 step  - the minimum expected size of the ellipse
c                           (input)
c                 sep   - the minimum expected separation between 
c                          neighboring ellipses. (input) 
c                 exc   - maximum allowed ratio of the major to minor 
c                           semi-axes. (input)
c                 npnt  - number of points for curve fitting, must
c                           be even. (input) 
c                 cols  - total number of columns in the image.
c                           (input)
c                 rows  - total number of rows in the image.
c                           (input)
c                 ximg  - x-coordinates of pixels in pixel.
c                           (input)
c                 yimg  - y-coordinates of pixels in pixel.
c                           (input)
c                 apixel  - area of pixel in pixel square.(input)
c
c   Required routines   - insoutBM, bound, center
c
c   Written by          - Miroslav Dujin
c   Modified by         - Mingshan Li
c
c   Note:  The program goes through nodes of the grid whose cell
c          size is <step> by <step> pixels. The node will be 
c          processed only if it is black and if it is separated 
c          by more than <sep> pixels from all previously extracted 
c          ellipses.
c          
c          The program searches for <npnt> points on the boundary
c          of the black area by drawing <npnt> straight lines from
c          the test point in <npnt> directions. These points are used
c          to reposistion the testing point more towards the center
c          of the black area. The process is then repeated to find
c          <npnt> fitting points on the boundary which are used to
c          fit an ellpise. If any one of the fitting points is found
c          to lie on the border of the image area, the ellipse will
c          be ignored.
c     
c          Program will discard all ellipses whose minor axis is 
c          smaller than <step> and all ellipses whose ratio of major
c          to minor axis is larger than <exc>.
c
c-----------------------------------------------------------------------
c     
      subroutine extractBM (img,ximg,yimg,a,b,rc,cc,th,area,
     *ne,step,sep,exc,rows,cols,apixel,dpi,sf1,sf2,affo,mnpart,vref) 
c
      implicit none
      integer            kp,maxr,maxc
      parameter         (kp = 200) ! max. no. of pts. on contour
      parameter         (maxr=1000,maxc=1000) ! image domain
c                                  Specifications for arguments
      integer           ne,step,sep,cols,rows,img(maxr,maxc),
     *                  rc(*),cc(*)
      real              a(*),b(*),th(*),area(*),exc,pi,dpi,sf1,sf2,
     *                  epara
      common            pi
c                                  Specifications for local variables
      integer           black,pxl,i,row,col,
     *                  lpxl,rpxl,upxl,dpxl,
     *                  col0,row0,ll
      real              a0,b0,rc0,cc0,th0,xcp,ycp,
     *                  ddc,ddr,dist,sqdist,conv,astep,
     *                  ximg(maxr,maxc),yimg(maxr,maxc),apixel,
     *                  totI0,totI1x,totI1y,totI2xx,totI2yy,totI2xy,
     *                  th2,th3(1000),ab0,factor,toti2xxact

      integer     break,j,lmax,array(maxr,maxc)
      integer   cright,c,cleft,np,aff(maxr,maxc),leftreached,rowsover,k,
     *             r,rtemp,ctemp,m,endofline,colend,affo(maxr,maxc)
      integer     rdata(150,10000),cdata(150,10000),count(150)
      real              I0,I1x,I1y,I2xx,I2yy,I2xy,dmax,value,vref
      integer     count1,count2,crep,rrep,u,l,local,pcount(5000),
     *           perimrow(150,500),perimcol(150,500),mnpart(1000)
      logical           insoutBM
      data              black/0/
c                                  First executable statement
      conv = 180./pi    ! conv to [deg]
      factor=1.0
      apixel=1.d0
c     
      ne = 0
      array=1
      count1=0
      do i=1,rows
         do j=1,cols
            aff(i,j)=0
            array(i,j)=img(i,j)
            if (array(i,j).eq.0) then
            count1=count1+1
            endif
         enddo
      enddo

      np=0
      aff=0
      rdata=0
      cdata=0
      count=0
      do i=1,rows
         do j=1,cols
            if (array(i,j).eq.0) then
               if (aff(i,j).eq.0) then
                  np=np+1
                  rtemp=i
                  ctemp=j
                  cleft=j
                  endofline=0
                  leftreached=0
                  rowsover=0
                  do m=0,200
                  if (endofline.ne.1) then
                     if (ctemp+m.le.cols) then
      if (array(rtemp,ctemp+m).eq.0.and.aff(rtemp,ctemp+m).ne.np) then
                     aff(rtemp,ctemp+m)=np
            if(affo(rtemp,ctemp+m).ne.0) mnpart(np)=affo(rtemp,ctemp+m)
                     count(np)=count(np)+1
                     rdata(np,count(np))=rtemp
                     cdata(np,count(np))=ctemp+m
                  else
                     cright=ctemp+m-1
                     endofline=1
                  endif
                  endif
                  endif
                  enddo
ccccccccccccccc first row is recorded ccccccccccccccccccccccccccccccccc

                  do r=rtemp+1,rtemp+200
                     leftreached=0
                     colend=0
                     if (rowsover.ne.1) then
                        if (r.lt.rows) then
                        if (array(r,cleft).eq.0) then
                           do k=1,200
                              if(leftreached.ne.1) then
                              cleft=cleft-1
                              if (array(r,cleft).eq.1) then
                                 leftreached=1
                                 cleft=cleft+1
                              endif
                              endif
                           enddo
                        
                        else
                           do k=cleft,cright
                              if (leftreached.ne.1) then
                              if (k.eq.cright) then
                                 rowsover=1
                                 exit
                              endif
                                 cleft=cleft+1
                                 if (array(r,cleft).eq.0) then
                                    leftreached=1
                                 endif
                              endif
                           enddo
                        endif
ccccc reached the first black pixel of next line.now record the line, 
ccccc  the new cleft and cright 
c     now start from cleft and scan until we find a white pixel.
                        
                        do c=cleft,cleft+200
                           if (colend.ne.1) then
                              if (c.lt.cols) then
                           if (array(r,c).eq.0.and.aff(r,c).ne.np) then
                              aff(r,c)=np
                             if(affo(r,c).ne.0) mnpart(np)=affo(r,c)
                              count(np)=count(np)+1
                              rdata(np,count(np))=r
                              cdata(np,count(np))=c
                           else
                              cright=c
                              colend=1
                           endif
                        endif
                        endif
                        enddo
                        endif
                     endif
                  enddo

cc  convex part is recorded. Now search for remaining part of particle
                  if (rowsover.eq.1) then
                     do local=1,200
                     do crep=cleft-150,cleft+150
                     do rrep=rtemp-150,rtemp+150
                        if (crep.gt.1.and.crep.lt.cols) then
                        if (rrep.gt.1.and.rrep.lt.rows) then
                        if (aff(rrep,crep).eq.np) then
                           do u=-1,1,2
                              do l=-1,1,2
        if (array(rrep+u,crep+l).eq.0.and.aff(rrep+u,crep+l).eq.0) then
           aff(rrep+u,crep+l)=np
        if(affo(rrep+u,crep+l).ne.0) mnpart(np)=affo(rrep+u,crep+l)
           count(np)=count(np)+1
           rdata(np,count(np))=rrep+u
           cdata(np,count(np))=crep+l
        endif
                              enddo
                           enddo
                        endif
                        endif
                        endif
                     enddo
                     enddo
                     enddo
                  endif
cc  particle scanning completed. now go to next particle
               endif
            endif
            if (np.gt.0) then
               value=count(np)*100.0/(cols*rows)
               if (value.le.vref) then 
                  do ll=1,count(np)
                     array(rdata(np,ll),cdata(np,ll))=1
                  enddo
                  rdata(np,:)=0
                  cdata(np,:)=0
                  count(np)=0
                  np=np-1
               endif
            endif
         enddo
      enddo
      PRINT *,'FINISHED SCANNING'
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      
      do i=1,np
         break=0
         I0 = 0.0               ! initialize the pixel's moment
         I1x = 0.0
         I1y = 0.0
         I2xx = 0.0
         I2yy = 0.0
         I2xy = 0.0
         totI0 = 0.0            ! initialize the particle's moment
         totI1x = 0.0
         totI1y = 0.0
         totI2xx = 0.0
         totI2yy = 0.0
         totI2xy = 0.0
         
         do j=1,count(i)
            totI0 = totI0+apixel         ! this pixel's moments
            totI1x = totI1x+apixel*rdata(i,j)
            totI1y = totI1y+apixel*cdata(i,j)
         enddo
         xcp = totI1y/totI0
         ycp = totI1x/totI0
         toti1x=0
         toti1y=0
          do j=1,count(i)
            totI2xx = totI2xx+(rdata(i,j)-ycp)**2
            totI2yy = totI2yy+(cdata(i,j)-xcp)**2
            totI2xy = totI2xy+((cdata(i,j)-xcp)*(-rdata(i,j)+ycp))
         enddo
         a0 = 0.0
         b0 = 0.0
         th0= 0.0
         th0=0.5*atan2(2*toti2xy,(toti2yy-toti2xx))
         if(th0.lt.0) th0=pi+th0
         toti2xxact=toti2xx*cos(th0)**2+toti2yy*sin(th0)**2-
     *                       2*toti2xy*sin(th0)*cos(th0)
         b0=sqrt(4*toti2xxact/toti0)
         a0=toti0/pi/b0
         cc0=xcp
         rc0=ycp

     
c     Calculate the center coordinates, semi-axes, and orientation of each
c     ellipse by comparing the 0-order, 1-order and 2nd-order moments 
c     between the actual particles and the simulation ellipses.

         if (epara.eq.1.0) then 
            print *,'removed due to epara'
            break=1 
         endif

c     Check if the ellipses overlap
         do k=1,ne
            ddc = cc0-cc(k)
            ddr = rc0-rc(k)
            sqdist = ddc*ddc+ddr*ddr+1.e-6   
            dist = sqrt (sqdist)
            ab0=(a0+b0+a(k)+b(k))/1.7            
c            ab0=b0+b(k)+sf2*sep
            if (dist .lt. ab0) then 
               a0=(factor*a0)
               b0=(factor*b0)
               a(k)=(factor*a(k))
               b(k)=(factor*b(k))
c               break=1  
c               print *,'removed due to overlap'
            endif
         enddo
c     Eliminate to excentric ones
         if (a0*1.0/b0 .ge. exc) then 
            print *,'removed due to high aspect ratio'
            break=1 
         endif
                  
c     Update info on ellipses
         if (break.ne.1) then
            ne = ne+1
            a(ne) = a0
            b(ne) = b0
            rc(ne) = rc0
            cc(ne) = cc0
            th(ne) = th0 
            th3(ne) = th2
            area(ne) = toti0
         endif
      enddo
      
      end
      
C =============================================================

c   routine name        - insoutBM
c                       
c-----------------------------------------------------------------------
c   
c   Computer            - HP/apollo/715/33
c
c   Latest revision     - Jan.18, 1996
c
c   Purpose             - Determine if the pixel belongs to 
c                            one of the previously found ellipses.
c
c   Usage               - function insoutBM (rows,cols,row,col,a,b,rc,
c                                            cc,th,ne,sep,sf)
c
c   Arguments    
c              insoutBM - flag that is true if pixel belongs to 
c                           one of the previously found ellipses
c                           and false otherwise. (output) 
c                row    - row coordinate of the pixel.
c                           (input)
c                col    - column coordinate of the pixel.
c                           (input)
c                a      - a real array of length ne containing 
c                           the major semi-axes of the previously
c                           found ellipses. (input)
c                b      - a real array of length ne containing
c                           the minor semi-axes of the previously 
c                           found ellipses. (input)
c                rc     - a real array of length ne containing
c                           row coordinates of the centers of the
c                           previou
c                cc     - a real array of length ne containing
c                           column coordinates of the centers of the
c                           previously found ellipses. (input) 
c                th     - a real array of length ne containing
c                           inclination angles of the previously
c                           found ellipses. (input)
c                ne     - number of previously found ellipses.
c                           (input)
c                sep    - number of pixels by which each of the 
c                           semi-axes will be extended before the
c                           decision is made. (input)
c
c   Required routines   - none
c
c   Written by          - Miroslav Dujin
c   Modified by         - Mingshan Li
c
c   Note:  Any point that lies within <sep> pixels from the
c          ellipse will be marked as if it lies inside. 
c
c-----------------------------------------------------------------------
c
      function insoutBM (rows,cols,row,col,a,b,rc,cc,th,ne,sep,sf)
      implicit none
c                                  Specifications for  arguments
      logical           insoutBM,T,F
      integer           rows,cols,row,col,ne,sep,rc(*),cc(*)
      real              a(*),b(*),th(*),sf
c                                  Specifications for local variables
      integer           i
      real              r,c,co,si,var1,var2,eqn
c
      T = .true.
      F = .false.  
c                                 First executable statement
      insoutBM = F
      if (ne .eq. 0) goto 20
c  Test for each ellipse
      do 10 i=1,ne    ! loop over each so far obtained ell.
        r = row-rc(i)
        c = col-cc(i)
        co = cos(th(i))
        si = sin(th(i))
        var1 = (c*co+r*si)/(a(i)+sf*sep)
        var2 = (-c*si+r*co)/(b(i)+sf*sep)
        eqn = var1*var1+var2*var2    ! ellipse equation
        if (eqn .le. 1) then
          insoutBM = T
          goto 20
        endif
 10   continue
c
 20   return
      end

C =============================================================

c   routine name        - ellipse
c                       
c-----------------------------------------------------------------------
c
c   Latest revision     - Jan.18, 1996
c
c   Purpose             - Determines the center coordinates, semi-axes
c                         and orientation of ellipses  by comparing 
c                         the 0-order,1-order and 2nd ordermoments 
c                         between the actual particles and the simulation
c                         ellipses.
c
c   Usage               -  call ellipse (totI0,totI1x,totI1y,
c     *                  totI2xx,totI2yy,
c     *                  a0,b0,cc0,rc0,th0,th2,epara)   
c
c   Arguments  
c                 I0    - particle's 0-order moment or area.(input)
c                 I1x   - particle's 1st-order moment.(input)
c                 I1y   - particle's 1st-order moment.(input)
c                 I2xx  - particle's 2nd-order moment.(input)
c                 I2yy  - particle's 2nd-order moment.(input)
c                 a    - length of the major semi-axes of the ellipses.
c                           (output)
c                 b    - length of the minor semi-axes of the ellipses.
c                           (output)
c                 xc   - x-(column) coordinate of the center of the
c                           ellipse. (output) 
c                 yc   - y-(row) coordinate of the center of the
c                           ellipse. (output) 
c                 th   - inclination angle of the ellipse by moment.
c                           (output)
c                 th2 -inclination angle of the ellipse for insoutBM.
c                           (output)
c
c   Required routines   - none
c
c   Written by          - Mingshan Li
c
c-----------------------------------------------------------------------
c
      subroutine ellipse (I0,I1x,I1y,I2xx,I2yy,I2xy,
     *                    a,b,xc,yc,th,th2,epara)   

      implicit real*8 (A-H,O-Z)  
c                                  Specifications for arguments
      real              I0,I1x,I1y,I2xx,I2yy,I2xy,I2xy1,I2xy2,
     *                  th2,epara,err1,err2

c                                  Specifications for local variables
      real              a,b,xc,yc,th,bc

c                                  First executable statement
c
      pi=4.0*atan(1.0)
      
      epara = 0.0
      
      a = 0.0
      b = 0.0
      xc = 0.0
      yc = 0.0
      th = 0.0
      
      th2 = 0.0
      
      xc = I1y*1.0/I0
      yc = I1x*1.0/I0
      
      c1=4*((I2xx+I2yy)*1.0/I0-(I1x**2+I1y**2)*1.0/I0**2)
      c2=I0**2/pi**2
      bc=abs(c1**2-4*c2)
      
               a1=sqrt(0.5*abs(c1+sqrt(bc)))
               a2=sqrt(0.5*abs(c1-sqrt(bc)))
      
      if(a1.ge.a2) then
         a=a1
      else
         a=a2
      endif
      b=I0/pi/a
      ang=4/(a**2-b**2)*((I2yy-I2xx)/I0-(I1y**2-I1x**2)/I0**2)

      if (abs(ang).gt.1.0) then
         if(ang.lt.-1) ang=-1
         if(ang.gt.1) ang=1
      else
         th=0.5*acos(ang)
      endif

      th2 = -th

      I2xy1=-1.0/8.0*pi*(a**2-b**2)*sin(2.0*th)+pi*a*b*xc*yc
      I2xy2=1.0/8.0*pi*(a**2-b**2)*sin(2.0*th)+pi*a*b*xc*yc

      err1=abs(I2xy-I2xy1)
      err2=abs(I2xy-I2xy2)

      if (err1.le.err2) then
         th = th
      else
         th = pi-th
      endif

 100  return
      end



C =============================================================

c   routine name        - formimgBM
c-----------------------------------------------------------------------
c   
c   Computer            - SUN Sparc station 2
c
c   Latest revision     - Jan.9, 1996
c
c   Purpose             - Reads bitmap PS image file and forms
c                           integer matrix IMG which represents
c                           image domain.
c                           - 0 corresponds to black pxl. and
c                             1 to the white one.  
c
c   Usage               - call formimgBM (unit,name,rows,cols,
c                           recl,rec0,img,ximg,yimg,dpi,apixel)
c
c   Arguments     unit  - unit number of the file containing the 
c                           data records of the image. 
c                           (input)
c                 name  - a strin variable containing the name
c                           of the input PS (bitmap) file
c                           (input) 
c                 rows  - total number of rows in the image.
c                           (output)
c                 cols  - total number of columns in the image.
c                         NOTE: must be divisible by four !
c                           (input)
c                 recl  - number of data records in a single
c                           data line. (input)
c                 rec0  - record number of the first data record.
c                           (input)
c                 img   - 2D (integer) matrix representing image 
c                           domain.
c                           (output)
c                 ximg  - x-coordinates of pixels in pixel.
c                           (output)
c                 yimg  - y-coordinates of pixels in pixel.
c                           (output)
c                 dpi   - pixels per inch.(input)
c                 apixel  - area of pixel in pixel square.(output)
c
c   Required routines   - hexbin
c
c   Written by          - Miroslav Dujin
c   Modified by         - Mingshan Li
c----------------------------------------------------------------------
c
      subroutine        formimgBM (unit,name,rows,cols,recl,rec0,
     *                             img,ximg,yimg,dpi,apixel)
      implicit none
      integer           maxr,maxc
      parameter         (maxr=1000,maxc=1000) ! image domain
c                                  Specifications for arguments
      integer           unit,rows,cols,recl,rec0,img(maxr,maxc),dpi  
c
      character*72      name
c                                  Specifications for local variables
      integer           rec0m1,recl1,record,recordr,tr,tc,ir,ic,
     *                  hex,r1,ncr,ic1,i1,i2,i3,i4    
      real              size,apixel,ximg(maxr,maxc),yimg(maxr,maxc)
c
c                                  First executable statement
c
      tr = 1      ! init.text row
      tc = 0      ! and col.
c
      rec0m1 = rec0-1
      recl1= recl+1  ! hex.digts/line including CR
c start reading HEX digits
 1    tc = tc+1      ! set curr.text col.
      if (tc.eq.recl1) then  ! end of cur.txt.line (CR)
        tr = tr+1  ! new curr.txt.row
        tc = 0
        goto 1
      endif
c
      record = rec0m1+tc+recl1*(tr-1)  ! cur.hex.digit seq.record #
      read (unit,100,rec=record) hex
 100  format (z1)
c
c now brake HEX into 4 pxls and fill image matrix
      recordr = record-rec0   ! rel.record # (for hex digts.)
      ncr = int(recordr/recl1)     ! # of CR's
      r1 = 4*(recordr-ncr)+1  ! seq.# of a 1st (of 4) pxl in img.file
c      
      ir = int(r1/cols)+1  ! curr.pxl.row in image matrix
      ic1 = r1-(ir-1)*cols ! curr.pxl.col.in image matrix (first of 4)
c
      call hexbin (hex,i1,i2,i3,i4)

c fill image matrix and obtain the coordinates of each pixel:

      size = 1.0          ! size of each pixel (square) in pixel
      apixel = size * size     ! area of each pixel in pixel square

      ic = ic1
      img (ir,ic) = i1
      ximg (ir,ic) = ic*size-0.5*size
      yimg (ir,ic) = ir*size-0.5*size
      ic = ic1+1
      img (ir,ic) = i2
      ximg (ir,ic) = ic*size-0.5*size
      yimg (ir,ic) = ir*size-0.5*size
      ic = ic1+2
      img (ir,ic) = i3
      ximg (ir,ic) = ic*size-0.5*size
      yimg (ir,ic) = ir*size-0.5*size
      ic = ic1+3
      img (ir,ic) = i4
      ximg (ir,ic) = ic*size-0.5*size
      yimg (ir,ic) = ir*size-0.5*size
c
      if ((ir.eq.rows).and.(ic.eq.cols)) then
        close(1)  ! txt.filec
        goto 20
      else
        goto 1
      endif
c
      close(2)  ! img.file finished
c
 20   return
      end

C =============================================================

c   routine name        - hexbin
c-----------------------------------------------------------------------
c   
c   Computer            - SUN Sparc station 2
c
c   Latest revision     - June 14, 1995
c
c   Purpose             - Converts HEX into its binary components: 
c                         i1*2^3+i2*2^2+i3*2^1+i4*2^0
c
c   Usage               - call subroutine hexbin (hex,i1,i2,i3,i4) 
c
c   Arguments     hex   - hex digit (decimal value 0-15)
c                           (input)
c                 i1-4  - binary components of HEX
c                           (output)
c
c   Required routines   - none
c
c   Written by          - Miroslav Dujin
c----------------------------------------------------------------------
      subroutine hexbin (hex,i1,i2,i3,i4) 
c
      implicit none
      integer hex,i1,i2,i3,i4
c
c Decomposition of hex digit (0-15) :
c
      goto (1,2,3,4,5,6,7,8,9,
     *      10,11,12,13,14,15,16), (hex+1)
c
c hex = 0:
 1    i1 = 0
      i2 = 0
      i3 = 0
      i4 = 0
      return
c hex = 1:
 2    i1 = 0
      i2 = 0
      i3 = 0
      i4 = 1
      return
c hex = 2:
 3    i1 = 0
      i2 = 0
      i3 = 1
      i4 = 0
      return
c hex = 3:
 4    i1 = 0
      i2 = 0
      i3 = 1
      i4 = 1
      return
c hex = 4:
 5    i1 = 0
      i2 = 1
      i3 = 0
      i4 = 0
      return
c hex = 5:
 6    i1 = 0
      i2 = 1
      i3 = 0
      i4 = 1
      return
c hex = 6:
 7    i1 = 0
      i2 = 1
      i3 = 1
      i4 = 0
      return
c hex = 7:
 8    i1 = 0
      i2 = 1
      i3 = 1
      i4 = 1
      return
c hex = 8: 
 9    i1 = 1
      i2 = 0
      i3 = 0
      i4 = 0
      return
c hex = 9:
 10   i1 = 1
      i2 = 0
      i3 = 0
      i4 = 1
      return
c hex = 10 (A):
 11   i1 = 1
      i2 = 0
      i3 = 1
      i4 = 0
      return
c hex = 11 (B):
 12   i1 = 1
      i2 = 0
      i3 = 1
      i4 = 1
      return
c hex = 12 (C):
 13   i1 = 1
      i2 = 1
      i3 = 0
      i4 = 0
      return
c hex = 13 (D):
 14   i1 = 1
      i2 = 1
      i3 = 0
      i4 = 1
      return
c hex = 14 (E):
 15   i1 = 1
      i2 = 1
      i3 = 1
      i4 = 0
      return
c hex = 15 (F):
 16   i1 = 1
      i2 = 1
      i3 = 1
      i4 = 1
      return
c
      end

C =============================================================

c   routine name        - center
c                       
c-----------------------------------------------------------------------
c   
c   Computer            - HP/apollo/715/33
c
c   Latest revision     - July 1, 1995
c
c   Purpose             - Determines the center of mass of a
c                           homogeneous polygon whose corner
c                           coordinates are given.
c
c   Usage               - call center (xc,yc,x,y,n)
c
c   Arguments    xc,yc  - coordinates of the center of mass of the
c                           polygon. (output)
c                x,y    - integer arrays of length 'n' containing 
c                           the corner points coordinates. (input)
c                n      - number of corner points
c
c   Required routines   - none
c
c   Written by          - Miroslav Dujin
c
c-----------------------------------------------------------------------
c
      subroutine center (xc,yc,x,y,n)
      implicit none
c                                  Specifications for arguments
      integer           x(*),y(*),n
      real              xc,yc
c                                  Specifications for local variables
      integer           j
      real              a,b,c,dx,dy,s,area,totarea,a2  
c                                  First executable statement
      xc = 0.                      ! initialization of center
      yc = 0.                      ! coordinates and total area
      totarea = 0.                 ! of the polygon
c
c                                  From one triangle to another
      do 10 j=2,n-1
         dx = x(j)-x(1)
         dy = y(j)-y(1)
         a = sqrt(dx*dx+dy*dy)
c
         dx = x(j+1)-x(1)
         dy = y(j+1)-y(1)
         b = sqrt(dx*dx+dy*dy)
c
         dx = x(j+1)-x(j)
         dy = y(j+1)-y(j)
         c = sqrt(dx*dx+dy*dy)
c
         s = 0.5*(a+b+c)
         a2 = s*(s-a)*(s-b)*(s-c)
         if (a2 .lt. 0.) a2 = 0.
         area = sqrt(a2)  ! area of elementary triangle
         totarea = totarea + area 
c
         xc = xc+(x(1)+x(j)+x(j+1))*area/3
         yc = yc+(y(1)+y(j)+y(j+1))*area/3
 10   continue
      xc = xc/totarea
      yc = yc/totarea
c
      return
      end

C =============================================================

c   routine name        - writePS
c                       
c-----------------------------------------------------------------------
c   
c   Computer            - HP/apollo/715/33
c
c   Latest revision     - August 22, 1994
c
c   Purpose             - Writes a PostScript file showing the 
c                           ellipses and the page frame.
c
c   Usage               - call writePS (name,cols,rows,fl,fw,
c                                         fx,fy,a,b,rc,cc,th,ne)
c
c   Arguments     name  - a string variable containing the name of
c                           the output postscript file. (input)
c                 cols  - total number of columns in the image.
c                           (input)
c                 rows  - total number of rows in the image.
c                           (input)
c                 dpi   - dots / inch
c                           (input)
c                 fl    - frame length in inches.
c                           (input)
c                 fw    - frame width in inches.
c                           (input)
c                 fx    - frame shift in the horizontal direction
c                           in inches. (input)
c                 fy    - frame shift in the vertical direction
c                           in inches. (input)
c                 a     - major semi-axis of the ellipse.
c                           (output)
c                 b     - minor semi-axis of the ellipse.
c                           (output)
c                 rc    - row coordinate of the center of the
c                           ellipse. (output)
c                 cc    - column coordinate of the center of the
c                           ellipse. (output)
c                 th    - angle (rad) between major axis and 
c                           the positive column direction. (output)
c                 ne    - number of ellipses
c
c   Required routines   - none
c
c   Written by          - Miroslav Dujin
c
c   Note:  The <rows> by <cols> pixels exactly fits inside the frame.
c
c-----------------------------------------------------------------------
c
      subroutine writePS1 (cols,rows,dpi,fl,fw,fx,fy,a,b,
     *                    rc,cc,th,ne,psize)
      implicit none
c                                  Specifications for arguments
      integer           cols,rows,ne,rc(*),cc(*)
      character*11     name
      real              a(*),b(*),th(*),pi,fl,fw,fx,fy,dpi,psize
      common            pi
c                                  Specifications for local variables
      integer           i,fontsize
      data              fontsize/25/
c                                  First executable statement
      name='ellipses.ps'
      open (101,file=name,status='unknown',
     *        access='sequential',form='formatted')
      write (101,50)  
 50   format('%!')
      write (101,*) fx*72+20,fy*72+550,' translate'
      write (101,*) 0.6*fw*75.2*(1.25)/cols,0.6*fl*75.2*
     *                                       (1.25)/rows,' scale'
      write (101,*) -90,' rotate'
c
c $      write (101,*) (fw*72-0.5)/cols,(fl*72-0.5)/rows,' scale'
c
      write (101,*) ' ' 
      write (101,*) '/Times-Roman findfont'
      write (101,*) fontsize,' scalefont'
      write (101,*) 'setfont'
      write (101,*) ' '
c                                  Draw the page frame
      write (101,*) 'newpath'
      write (101,*) '0 0 moveto' ! cut blank scanned part
      write (101,*) cols+0.5*psize*dpi,' 0 lineto'
      write (101,*) cols+0.5*psize*dpi,rows+0.5*psize*dpi,
     *     ' lineto'
      write (101,*) '0 ',rows+0.5*psize*dpi,' lineto'
      write (101,*) 'closepath'
      write (101,*) 'stroke'
      write (101,*) ' '
c                                  Draw ellipses
      do 10 i=1,ne
         write (101,*) 'gsave'
         write (101,*) cc(i)+0.25*psize*dpi,rows-rc(i)+
     &        0.25*psize*dpi,' translate'
c change origin from left top to left bottom
         write (101,*) -0.5*fontsize,-0.5*fontsize,' moveto'
c $        write (101,800) '(',i,') show'
         write (101,*) th(i)/pi*180,' rotate'
         write (101,*) '1. ',b(i)/a(i),' scale'
         write (101,*) 'newpath'
C         a(i) = a(i)*dpi   ! convert to pxl's                  
         write (101,*) '0 0 ',a(i),' 0 360 arc'
         write (101,*) 'stroke'
         write (101,*) 'grestore'
         write (101,*) ' '
 10   continue
c                                  Trailer
      write (101,*) 'showpage'
c
      close(101)    
 800  format(1x,a1,i3,a6)
      return
      end
C =============================================================

c   routine name        - writePS
c                       
c-----------------------------------------------------------------------
c   
c   Computer            - HP/apollo/715/33
c
c   Latest revision     - August 22, 1994
c
c   Purpose             - Writes a PostScript file showing the 
c                           ellipses and the page frame.
c
c   Usage               - call writePS (name,cols,rows,fl,fw,
c                                         fx,fy,a,b,rc,cc,th,ne)
c
c   Arguments     name  - a string variable containing the name of
c                           the output postscript file. (input)
c                 cols  - total number of columns in the image.
c                           (input)
c                 rows  - total number of rows in the image.
c                           (input)
c                 dpi   - dots / inch
c                           (input)
c                 fl    - frame length in inches.
c                           (input)
c                 fw    - frame width in inches.
c                           (input)
c                 fx    - frame shift in the horizontal direction
c                           in inches. (input)
c                 fy    - frame shift in the vertical direction
c                           in inches. (input)
c                 a     - major semi-axis of the ellipse.
c                           (output)
c                 b     - minor semi-axis of the ellipse.
c                           (output)
c                 rc    - row coordinate of the center of the
c                           ellipse. (output)
c                 cc    - column coordinate of the center of the
c                           ellipse. (output)
c                 th    - angle (rad) between major axis and 
c                           the positive column direction. (output)
c                 ne    - number of ellipses
c
c   Required routines   - none
c
c   Written by          - Miroslav Dujin
c
c   Note:  The <rows> by <cols> pixels exactly fits inside the frame.
c
c-----------------------------------------------------------------------
c
      subroutine writePS2 (name,cols,rows,dpi,fl,fw,fx,fy,a,b,
     *                    rc,cc,th,ne,psize)
      implicit none
c                                  Specifications for arguments
      integer           cols,rows,ne,rc(*),cc(*)
      character*(*)     name
      real              a(*),b(*),th(*),pi,fl,fw,fx,fy,dpi,psize
      common            pi
c                                  Specifications for local variables
      integer           i,fontsize
      data              fontsize/25/
c                                  First executable statement
      open (101,file=name,status='unknown',
     *        access='sequential',form='formatted')
      write (101,50)  
 50   format('%!')
      write (101,*) (fx+0.2)*72,fy*72+470,' translate'
      write (101,*) 0.8*fw*75.2*(1.25)/cols,
     *     0.8*fl*75.2*(1.25)/rows,' scale'
      write (101,*) -90,' rotate'
c
      write (101,*) ' ' 
      write (101,*) '/Times-Roman findfont'
      write (101,*) fontsize,' scalefont'
      write (101,*) 'setfont'
      write (101,*) ' '
c                                  Draw the page frame
      write (101,*) 'newpath'
      write (101,*) '0 0 moveto' ! cut blank scanned part
      write (101,*) cols+0.5*psize*dpi-140,' 0 lineto'
      write (101,*) cols+0.5*psize*dpi-140,rows+0.5*psize*dpi,
     *     ' lineto'
      write (101,*) '0 ',rows+0.5*psize*dpi,' lineto'
      write (101,*) 'closepath'
      write (101,*) 'stroke'
      write (101,*) ' '
c                                  Draw ellipses
      do 10 i=1,ne
         write (101,*) 'gsave'
         write (101,*) cc(i)+0.25*psize*dpi-70,rows-rc(i)+
     &        0.25*psize*dpi,' translate'
c change origin from left top to left bottom
         write (101,*) -0.5*fontsize,-0.5*fontsize,' moveto'
         write (101,*) th(i)/pi*180,' rotate'
         write (101,*) '1. ',b(i)/a(i),' scale'
         write (101,*) 'newpath'
         write (101,*) '0 0 ',a(i),' 0 360 arc'
         write (101,*) 'stroke'
         write (101,*) 'grestore'
         write (101,*) ' '
 10   continue
c                                  Trailer
      write (101,*) 'showpage'
c
      close(101)    
 800  format(1x,a1,i3,a6)
      return
      end

C =============================================================

c   routine name        - writePS
c                       
c-----------------------------------------------------------------------
c   
c   Computer            - HP/apollo/715/33
c
c   Latest revision     - August 22, 1994
c
c   Purpose             - Writes a PostScript file showing the 
c                           ellipses and the page frame.
c
c   Usage               - call writePS (name,cols,rows,fl,fw,
c                                         fx,fy,a,b,rc,cc,th,ne)
c
c   Arguments     name  - a string variable containing the name of
c                           the output postscript file. (input)
c                 cols  - total number of columns in the image.
c                           (input)
c                 rows  - total number of rows in the image.
c                           (input)
c                 dpi   - dots / inch
c                           (input)
c                 fl    - frame length in inches.
c                           (input)
c                 fw    - frame width in inches.
c                           (input)
c                 fx    - frame shift in the horizontal direction
c                           in inches. (input)
c                 fy    - frame shift in the vertical direction
c                           in inches. (input)
c                 a     - major semi-axis of the ellipse.
c                           (output)
c                 b     - minor semi-axis of the ellipse.
c                           (output)
c                 rc    - row coordinate of the center of the
c                           ellipse. (output)
c                 cc    - column coordinate of the center of the
c                           ellipse. (output)
c                 th    - angle (rad) between major axis and 
c                           the positive column direction. (output)
c                 ne    - number of ellipses
c
c   Required routines   - none
c
c   Written by          - Miroslav Dujin
c
c   Note:  The <rows> by <cols> pixels exactly fits inside the frame.
c
c-----------------------------------------------------------------------
c
      subroutine writePS3 (namei,name,cols,rows,dpi,fl,fw,fx,fy,a,b,
     *                    rc,cc,th,ne,psize)
      implicit none
c                                  Specifications for arguments
      integer           cols,rows,ne,rc(*),cc(*)
      integer           i1,i2,i3,i4,k
      character*72      namei,name
      character*72      line,char
      character*1       ch(600,300)
      real              a(*),b(*),th(*),pi,fl,fw,fx,fy,dpi,psize
      common            pi
c                                  Specifications for local variables
      integer           i,fontsize
      data              fontsize/25/
c                                  First executable statement

      open(108,file=namei,status='old',
     *         access='sequential',form='formatted')

      open (101,file=name,status='unknown')


      write (101,601)  
 601  format('%!PS-Adobe-2.0 EPSF-2.0')
      write (101,602)  
 602  format('%%Title: /homea/limi/image/4c/4c_50_0.ps')
      write (101,603)  
 603  format('%%Creator: XV Version 3.10a  Rev: 12/29/94  -  
     &by John Bradley')
      write (101,604)  
 604  format('%%BoundingBox: 66 156 546 636')
      write (101,605)  
 605  format('%%Pages: 1')
      write (101,606)  
 606  format('%%DocumentFonts:')
      write (101,607)  
 607  format('%%EndComments')
      write (101,608)  
 608  format('%%EndProlog')
      write (101,609)  
 609  format('')
      write (101,610)  
 610   format('%%Page: 1 1')
      write (101,611)  
 611  format('')
      write (101,612)  
 612  format('% remember original state')
      write (101,613)  
 613  format('/origstate save def')
      write (101,614)  
 614  format('')
      write (101,615)  
 615  format('% build a temporary dictionary')
      write (101,616)  
 616  format('20 dict begin')
      write (101,617)  
 617  format('')
      write (101,618)  
 618  format('% define string to hold a scanline worth of data')
      write (101,619)  
 619  format('/pix 60 string def')
      write (101,620)  
 620   format('')
      write (101,621)  
 621  format('% define space for color conversions')
      write (101,622)  
 622  format('/grays 480 string def  % space for gray scale line')
      write (101,623)  
 623  format('/npixls 0 def')
      write (101,624)  
 624  format('/rgbindx 0 def')
      write (101,625)  
 625  format('')
      write (101,626)  
 626  format('% lower left corner')
      write (101,627)  
 627  format('66 156 translate')
      write (101,628)  
 628  format('')
      write (101,629)  
 629  format('% size of image (on paper, in 1/72inch coords)')
      write (101,630)  
 630   format('480.02400 480.02400 scale')
      write (101,631)  
 631  format('')
      write (101,632)  
 632  format('% dimensions of data')
      write (101,633)  
 633  format('480 480 1')
      write (101,634)  
 634  format('')
      write (101,635)  
 635  format('% mapping matrix')
      write (101,636)  
 636  format('[480 0 0 -480 0 480]')
      write (101,637)  
 637   format('')
      write (101,638)  
 638  format('{currentfile pix readhexstring pop}')
      write (101,639)  
 639  format('image')


      do 20 i=1,39
         read (108,810) line
 20   continue

      do 22 i1=1,480  ! read image hex. data

         read (108,810) char
         do 23 i2=1,72
            ch(i1,i2)=char(i2:i2)
 23      continue
         write (101,830)(ch(i1,k),k=1,72)
         read (108,811) char
         do 24 i3=1,48
            ch(i1,72+i3)=char(i3:i3)
 24      continue
         write (101,830)(ch(i1,72+k),k=1,48)
        
 22   continue
      
      close(108)

 810  format(a72)
 811  format(a48)
 830  format(72(a1))
 850  format(16(a1))

      write (101,730)  
 730  format('')
      write (101,731)  
 731  format('')
      write (101,734)  
 734  format('% stop using temporary dictionary')
      write (101,735)  
 735  format('end')
      write (101,736)  
 736  format('')
      write (101,737)  
 737  format('% restore original state')
      write (101,738)  
 738  format('origstate restore')
      write (101,739)  
 739  format('')
      write (101,740)  
 740  format('%%Trailer')
      write (101,741)  
 741  format('')
      write (101,742)  
 742  format('')


      write (101,50)  
 50   format('%!')
      write (101,*) 66,156,' translate'
      write (101,*) 0.6*fw*75.2*(1.25)/cols,
     *     0.6*fl*75.2*(1.25)/rows,' scale'
      write (101,*) ' ' 
      write (101,*) '/Times-Roman findfont'
      write (101,*) fontsize,' scalefont'
      write (101,*) 'setfont'
      write (101,*) ' '
c                                  Draw the page frame
      write (101,*) 'newpath'
      write (101,*) '0 0 moveto' ! cut blank scanned part
      write (101,*) cols+0.5*psize*dpi,' 0 lineto'
      write (101,*) cols+0.5*psize*dpi,rows+0.5*psize*dpi,
     *     ' lineto'
      write (101,*) '0 ',rows+0.5*psize*dpi,' lineto'
      write (101,*) 'closepath'
      write (101,*) 'stroke'
      write (101,*) ' '
c                                  Draw ellipses
      do 10 i=1,ne
         write (101,*) 'gsave'
         write (101,*) cc(i)+0.25*psize*dpi,rows-rc(i)+
     &        0.25*psize*dpi,' translate'
c change origin from left top to left bottom
         write (101,*) -0.5*fontsize,-0.5*fontsize,' moveto'
         write (101,*) th(i)/pi*180,' rotate'
         write (101,*) '1. ',b(i)/a(i),' scale'
         write (101,*) 'newpath'
         write (101,*) '0 0 ',a(i),' 0 360 arc'
         write (101,*) 'stroke'
         write (101,*) 'grestore'
         write (101,*) ' '
 10   continue
c                                  Trailer
      write (101,*) 'showpage'
c
      close(101)    
 800  format(1x,a1,i3,a6)
      return
      end

C =============================================================

c   routine name        - woutdat1
c                       
c-----------------------------------------------------------------------
c   
c   Computer            - Sun Sparc station 2
c
c   Latest revision     - June 30, 1994
c
c   Purpose             - Writes an output data file with recognized
c                           ellipse data as an inpit file in mesh
c                           generation.
c
c   Usage               - call woutdat1 (name,rows,dpi,ne,a,b,rc,cc,th,
c    *area,step0,sep0,exc,sf)
c
c   Arguments     name  - a string variable containing the name of
c                           the output data file. (input)
c                 rows  - total number of rows in the image.
c                           (input)
c                 dpi   - dots/inch 
c                           (input)
c                 ne    - number of ellipses
c                           (input)
c                 a     - major semi-axis of the ellipse.
c                           (input)
c                 b     - minor semi-axis of the ellipse.
c                           (input)
c                 rc    - row coordinate of the center of the
c                           ellipse. (input)
c                 cc    - column coordinate of the center of the
c                           ellipse. (input)
c                 th    - angle (rad) between major axis and 
c                           the positive column direction. (input)
c
c   Required routines   - none
c
c   Written by          - Miroslav Dujin
c
c   Note: File header is contains default values which can be 
c         changed before going into mesh generation 
c
c-----------------------------------------------------------------------
c
      subroutine woutdat1 (name,rows,cols,dpi,ne,a,b,cc,rc,th,hs,vs,psize,npv
     *                      ,connect,xypolyv,polyflag)
c
      implicit none
      integer k
      parameter (k=1000) ! max.no.of incls.
c                                  Specifications for arguments
      integer           ne,rc(*),cc(*),rows,cols
      integer         npv(150),connect(1000,100),polyflag(150)
      real             xypolyv(1000,2)
      character*(*)     name
      real              a(*),b(*),th(*),pi,hs,vs,dpi,psize
      common            pi
c                                  Specifications for local variables
      integer           i,tmp2,m
      real              conv1,conv2,xc(k),yc(k),theta(k),
     *                  tmp,origin

c                                  First executable statement
      conv1 = 180./pi   ! [rad] -> [deg]
      conv2 = 1./dpi  ! [pxl] -> [in]
      origin=0.0
      hs=cols/dpi
      vs=rows/dpi
c                                  
      open (107,file=name,status='unknown')
c
      write (107,*) 'No.of internal openings within meshed region:'
      write (107,*) '0'
      write (107,*) 'Number of region boundary nodes:'
      write (107,*) '4'    ! rect.frame
      write (107,*) 'Bound.node cords [in] (COUNTERCLOCKWISE):'
      write (107,*) origin,origin
      write (107,*) hs+0.5*psize,origin
      write (107,*) hs+0.5*psize,vs+0.5*psize
      write (107,*) origin,vs+0.5*psize
      write (107,*) 'Horiz.size of region new frame [in]:'
      write (107,*) hs+0.5*psize
      write (107,*) 'Vert.size of region new frame [in]:'
      write (107,*) vs+0.5*psize
      write (107,*) 'Frame shift in the horiz. dir. [in]:' 
      write (107,*) origin
      write (107,*) 'Frame shift in the vert. dir. [in]:'
      write (107,*) origin
      write (107,*) 'Number of extracted ellipses:'
      write (107,*) ne
      write (107,800)  
 800  format('num',3x,'a(in)',4x,'b(in)',3x,'xc(in)',3x,
     *'yc(in)',2x,'theta(dgr)',3x,'i g n o r e',/,65('-'))
      do 1  i = 1,ne
        a(i) = a(i)/dpi   ! convert to inches
        b(i) = b(i)/dpi
        xc(i) = cc(i)*conv2+0.25*psize
        yc(i) = (rows-rc(i))*conv2+0.25*psize
        theta(i) = th(i)*conv1
        tmp = 0.
        tmp2=npv(i)
        write (107,810) i,a(i),b(i),xc(i),yc(i),theta(i),tmp,tmp2
 810    format (i3,4(2x,f6.3,1x),3x,f6.1,4x,f3.1,2x,i3)
        if(polyflag(i).ne.0) then
           write(107,*) (xypolyv(connect(i,m),1)/dpi,
     *              xypolyv(connect(i,m),2)/dpi,m=1,npv(i))
        endif
 1    continue
c
      close (107)
c
      return
      end


C =============================================================

c   routine name        - woutdat2
c                       
c-----------------------------------------------------------------------
c   
c   Computer            - Sun Sparc station 2
c
c   Latest revision     - June 30, 1994
c
c   Purpose             - Writes an output data file with recognized
c                           ellipse data as an inpit file in mesh
c                           generation.
c
c   Usage               - call woutdat2 (name,rows,dpi,ne,a,b,rc,cc,th,
c    *area,step0,sep0,exc,sf)
c
c   Arguments     name  - a string variable containing the name of
c                           the output data file. (input)
c                 rows  - total number of rows in the image.
c                           (input)
c                 dpi   - dots/inch 
c                           (input)
c                 ne    - number of ellipses
c                           (input)
c                 a     - major semi-axis of the ellipse.
c                           (input)
c                 b     - minor semi-axis of the ellipse.
c                           (input)
c                 rc    - row coordinate of the center of the
c                           ellipse. (input)
c                 cc    - column coordinate of the center of the
c                           ellipse. (input)
c                 th    - angle (rad) between major axis and 
c                           the positive column direction. (input)
c
c   Required routines   - none
c
c   Written by          - Miroslav Dujin
c
c     Note: File header is contains default values which can be 
c         changed before going into mesh generation 
c
c-----------------------------------------------------------------------
c
      subroutine woutdat2 (name,rows,dpi,ne,a,b,cc,rc,th,area
     *                    ,step,sep,exc,sf1,sf2,hs,vs,psize)
c
      implicit none
      integer k
      parameter (k=1000) ! max.no.of incls.
c                                  Specifications for arguments
      integer           ne,rc(*),cc(*),rows
      character*(*)     name
      real              a(*),b(*),th(*),area(*),pi,hs,vs,
     *                  dpi,step,sep,sf1,sf2,exc,psize
      common            pi
c                                  Specifications for local variables
      integer           i
      real              conv1,conv2,theta(k),origin,
     *                  asp(k),avearea,aveleng,aveasp,aveori,
     *                  totarea,totleng,totasp,totori,
     *                  awin1,afrac1,awin2,afrac2

c                                  First executable statement
      conv1 = 180./pi   ! [rad] -> [deg]
      conv2 = 1./dpi  ! [pxl] -> [in]
      origin=0.0
c                                  
      open (109,file=name,status='unknown')
c

      write (109,*) 'Input data:'
      write (109,*) '        '
      write (109,*) 'Approx.inner circle diam.of'
      write (109,*) '     the smallest cell [mm]:'
      write (109,*) step
      write (109,*) '1/3 size.of the near bound.largest particle[mm]:'
      write (109,*) psize*25.4
      write (109,*) 'The smallest allowable separation'
      write (109,*) '            between ellipses [mm]:'
      write (109,*) sep
      write (109,*) 'Maximum allowable ratio of major to minor axis:'
      write (109,*) exc
      write (109,*) 'Shape complex factor for scaning new pixel:'
      write (109,*) sf1
      write (109,*) 'Shape complex factor for forming new ellipse:'
      write (109,*) sf2
      write (109,*) '         '

      write (109,*) 'Output data:'
      write (109,*) '        '
      write (109,*) 'Number of extracted ellipses:'
      write (109,*) ne
      write (109,*) '         '

      write (109,800)  
 800  format('num',1x,'aspect ratio',1x,'length(in)',
     &1x,'area(sq.in)',/,57('-'))
      do 1  i = 1,ne
        asp(i) = a(i)/b(i)
        area(i) = area(i)/dpi/dpi
        write (109,810) i,asp(i),a(i),area(i)
 810    format (i3,2x,f6.3,1x,6x,f6.3,5x,f6.3)
 1    continue
c

C output average values
      totarea = 0.0
      totleng = 0.0
      totasp = 0.0
      totori = 0.0
c
      write (109,*)  
      write (109,*)  
      write (109,600)  
 600  format('particle ave values:')
      write (109,*)  
      write (109,605)  
 605  format('area(sq.in)',2x,'length(in)',2x,'aspect ratio',
     &2x,'ori(deg)',/,57('-'))
      do 11  i = 1,ne
         totarea = totarea+area(i)
         totleng = totleng+a(i)
         totasp = totasp+asp(i)
         theta(i) = th(i)*conv1
         totori = totori+theta(i)
 11   continue
      avearea = totarea/ne
      aveleng = totleng/ne
      aveasp = totasp/ne
      aveori = totori/ne

      write (109,610) avearea,aveleng,aveasp,aveori
 610  format (1x,f6.3,1x,6x,f6.3,6x,f6.3,6x,f8.2)
c
      write (109,*)  
      write (109,630)  
 630  format('total particle area(sq.inch):')
      write (109,640) totarea
 640  format (1x,f9.3,1x)
c
      awin1=(hs-140/dpi)*vs
      write (109,650)  
 650  format('window area(sq.inch):')
      write (109,640) awin1

      afrac1=totarea/awin1
      write (109,660)  
 660  format('simulated particle area fraction:')
      write (109,670) afrac1
 670  format (1x,f10.6,1x)

c
      awin2=(hs+0.5*psize-140/dpi)*(vs+0.5*psize)
      write (109,680)  
 680  format('enlarged window area(sq.inch):')
      write (109,640) awin2

      afrac2=totarea/awin2
      write (109,690)  
 690  format('simulated particle area fraction for enlarged window:')
      write (109,670) afrac2

      close (109)
c
      return
      end

