#!/usr/bin/env python

##############################################################################
##                                                                          ##
##                                                                          ##
##                                                                          ##
##                   #####    ##     ##      ######     ##                  ##
##                 ##   ##   ###    ###     ##   ##    ##                   ##
##                ##        ## ##  ####    #######    ##                    ##
##               ##   ##   ##  ##### ##   ##  ##     ##                     ##
##               #####    ##    ###  ##  ##   ##    #######                 ##
##                                                                          ##
##                                                                          ##
##                      Ohio Supercomputer Center                           ##
##                                                                          ##
##                              08/23/06                                    ##
##                                                                          ##
##                                                                          ##
##                          Dr. Somnath Ghosh                               ##
##                                and                                       ##
##                              Hu Chao                                     ##
##                                                                          ##
##                  Department of Mechanical Engineering                    ##
##                       The Ohio State University                          ##
##                                                                          ##
##                                                                          ##
##                                                                          ##
##   Purpose:                                                               ##
##      Functions called from CMRL.py to create actors for delauney, or     ##
##      plots, or create images.                                            ##
##                                                                          ##
##                                                                          ##
##   Functions:                                                             ##
##         SavePS():                                                        ##
##         SaveJPEG():                                                      ##
##         SaveTIFF():                                                      ##
##         SavePNG():                                                       ##
##              Saves rendered window in the indicated image format.        ##
##                                                                          ##
##         makeLegends():                                                   ##
##              Creates the legend from the matrixActor.                    ##
##                                                                          ##
##         prepareData():                                                   ##
##              Intermediate call before calling the file reading functions ##
##              in readFile.py.                                             ##
##                                                                          ##
##                                                                          ##
##                                                                          ##
##                                                                          ##
##############################################################################

import vtk
from vtk.util.colors import *
from readfile import *

def SavePS(framenum, basenum, name, rW):
    adjust = framenum + basenum
    w2i = vtk.vtkWindowToImageFilter()
    w2i.SetInput(rW)
    writer = vtk.vtkPostScriptWriter()
    writer.SetInput(w2i.GetOutput())

    if adjust < 10:
       ImageFileName = name + ".000" + str(adjust) + ".ps"
    elif adjust < 100:
       ImageFileName = name + ".00" + str(adjust) + ".ps"
    elif adjust < 1000:
       ImageFileName = name + ".0" + str(adjust) + ".ps"
    elif adjust < 10000:
       ImageFileName = name + "." + str(adjust) + ".ps"
    #print ImageFileName
    writer.SetFileName(ImageFileName)
    writer.Write()
    print "Saved " + ImageFileName

def SaveJPEG(framenum, basenum, name, rW):
    adjust = framenum + basenum
    w2i = vtk.vtkWindowToImageFilter()
    w2i.SetInput(rW)
    writer = vtk.vtkJPEGWriter()
    writer.SetInput(w2i.GetOutput())

    if adjust < 10:
       ImageFileName = name + ".000" + str(adjust) + ".jpg"
    elif adjust < 100:
       ImageFileName = name + ".00" + str(adjust) + ".jpg"
    elif adjust < 1000:
       ImageFileName = name + ".0" + str(adjust) + ".jpg"
    elif adjust < 10000:
       ImageFileName = name + "." + str(adjust) + ".jpg"
##     print ImageFileName
    writer.SetFileName(ImageFileName)
    writer.SetQuality(100)
    writer.ProgressiveOff()
    writer.Write()
    print "Saved " + ImageFileName

def SaveTIFF(framenum, basenum, name, rW):
    adjust = framenum + basenum
    w2i = vtk.vtkWindowToImageFilter()
    w2i.SetInput(rW)
    writer = vtk.vtkTIFFWriter()
    writer.SetInput(w2i.GetOutput())

    if adjust < 10:
       ImageFileName = name + ".000" + str(adjust) + ".tiff"
    elif adjust < 100:
       ImageFileName = name + ".00" + str(adjust) + ".tiff"
    elif adjust < 1000:
       ImageFileName = name + ".0" + str(adjust) + ".tiff"
    elif adjust < 10000:
       ImageFileName = name + "." + str(adjust) + ".tiff"
    #print ImageFileName
    writer.SetFileName(ImageFileName)
    writer.Write()
    print "Saved " + ImageFileName
    
def SavePNG(framenum, basenum, name, rW):
    adjust = framenum + basenum
    w2i = vtk.vtkWindowToImageFilter()
    w2i.SetInput(rW)
    writer = vtk.vtkPNGWriter()
    writer.SetInput(w2i.GetOutput())

    if adjust < 10:
       ImageFileName = name + ".000" + str(adjust) + ".png"
    elif adjust < 100:
       ImageFileName = name + ".00" + str(adjust) + ".png"
    elif adjust < 1000:
       ImageFileName = name + ".0" + str(adjust) + ".png"
    elif adjust < 10000:
       ImageFileName = name + "." + str(adjust) + ".png"
    #print ImageFileName
    writer.SetFileName(ImageFileName)
    writer.Write()
    print "Saved " + ImageFileName
    
def makeLegends(matrixActor, matvarkey, dictIncl):
    matMesh = matrixActor.GetMapper()

    scalarBar = vtk.vtkScalarBarActor()
    scalarBar.SetLookupTable(matMesh.GetLookupTable())
    scalarBar.SetTitle(matvarkey)
    scalarBar.GetPositionCoordinate().SetCoordinateSystemToNormalizedViewport()
    scalarBar.GetPositionCoordinate().SetValue(0.80, 0.03)
    scalarBar.SetOrientationToVertical()
    scalarBar.SetNumberOfLabels(5)
    scalarBar.SetWidth(0.1)
    scalarBar.SetHeight(1.1)
    ltprop=scalarBar.GetLabelTextProperty()
    ltprop.SetFontSize(2)
    ltprop.SetFontFamilyToArial()
    ltprop.SetColor(0, 0, 0)
    ltprop.BoldOn()
    ltprop.ItalicOff()
    scalarBar.SetLabelTextProperty(ltprop)
    ttprop=scalarBar.GetTitleTextProperty()
    ttprop.SetFontSize(1)
    ttprop.SetFontFamilyToCourier()
    ttprop.SetColor(0, 0, 0)
    ttprop.BoldOff()
    ttprop.ItalicOff()
    scalarBar.SetTitleTextProperty(ttprop)

    return scalarBar

def prepareData(matfilename, inclfilename, framenum, doMatrix, doInclusion, incRenTubes, incRenPolys, matvarkey, inclvarkey, Icolor, Vmax, Vmin):
    matdictVals = {}
    dictIncl = {}
    incldictVals = {}
    dictIncDlny = {}
    dictHole = {}
    #Icolor=1
    # Reading the matrix data
##     matFile = open(os.path.normpath(filename + ".mat"))
##    print Icolor
    matFile = open(matfilename)
    matline = matFile.readline()
    matdata = string.split(matline)
    while (matdata[1][:3] == "var"):
        matdictVals[matdata[1]] = [matdata[2], matdata[3]]

        matline = matFile.readline()
        matdata = string.split(matline)

    # Reading the inclusion data
##     inclFile = open(os.path.normpath(filename + ".incl"))

    inclFile = open(inclfilename)
    inclline = inclFile.readline()
    incldata = string.split(inclline)
    while (incldata[1][:3] == "var"):
        incldictVals[incldata[1]] = [incldata[2], incldata[3]]

        inclline = inclFile.readline()
        incldata = string.split(inclline)

    while (matline):
        if (matdata[1] == 'IST' and int(matdata[2]) == framenum):
            break
        matline = matFile.readline()
        matdata = string.split(matline)

    while (inclline):
        if (incldata[1] == 'IST' and int(incldata[2]) == framenum):
            break
        inclline = inclFile.readline()
        incldata = string.split(inclline)

    if (doMatrix == 1):
        # File should be pointing to the very first IST
        if (matdata[1] == "IST"):
            # Call the function for reading matrix format. Returns an actor.
            matrixActor, edgeActor, matrxdlny, xmax, xmin, ymax, ymin = \
                         newMatFormat(matdictVals, matvarkey, matFile, Icolor, Vmax, Vmin)

    if (doInclusion == 1):
        # File should be pointing to the very first IST
        if (incldata[1] == "IST"):
            # Should be NUMINC
            inclline = inclFile.readline()
            incldata = string.split(inclline)
            #print "starting Incl line " + inclline

            if (incldata and incldata[1] == 'NUMINC'):

                numincls = int(incldata[2])
                #print "NUMINC is " + str(numincls)

                if (incRenTubes == 1):
                    #print "Rendering tubes"
                    for j in range(numincls):
                        name = "incl" + str(j)
                        holename = "hole" + str(j)
                        # Call the function for reading inclusion format.
                        success, meshActor, holeActor = \
                                 newIncTubeFormat(incldictVals, inclvarkey, \
                                                  inclFile, Icolor, Vmax, Vmin)
                        if (success == 1):
                            dictIncl[name] = meshActor
                            dictHole[holename] = holeActor
                        
                elif (incRenPolys == 1):
                    #print "Rendering polys"
                    for j in range(numincls):
                        name = "incl" + str(j)
                        holename = "hole" + str(j)
                        
                        # Call the function for reading inclusion format.
                        success, meshActor, incdlny, holeActor = \
                                 newIncPolyFormat(incldictVals, inclvarkey, \
                                                  inclFile, Icolor, Vmax, Vmin)
                        
                        dictIncl[name] = meshActor
                        dictIncDlny[name] = incdlny

                        if (success == 1):
                            dictHole[holename] = holeActor
                        
                else:
                    #print "Rendering tubes"
                    for j in range(numincls):
                        name = "incl" + str(j)
                        holename = "hole" + str(j)
                        # Call the function for reading inclusion format.
                        success, meshActor, holeActor = \
                                 newIncTubeFormat(incldictVals, inclvarkey, \
                                                  inclFile, Icolor, Vmax, Vmin)
                        if (success == 1):
                            dictIncl[name] = meshActor
                            dictHole[holename] = holeActor
                    
            else:
                print "We have a problem at initial reading of inclusion"

    # Close the files
    matFile.close()
    inclFile.close()


    return matrixActor, dictIncl, matrxdlny, dictIncDlny, dictHole, xmax, xmin, ymax, ymin

def prepPlot(probe):
    appendF = vtk.vtkAppendPolyData()

    xyplot = vtk.vtkXYPlotActor()
    xyplot.GetPositionCoordinate().SetCoordinateSystemToNormalizedViewport()
    xyplot.GetPositionCoordinate().SetValue(0.05, 0.05, 0)
    xyplot.GetPosition2Coordinate().SetCoordinateSystemToNormalizedViewport()
    #relative to Position
    xyplot.GetPosition2Coordinate().SetValue(0.95, 0.95, 0) 
    xyplot.SetXValuesToArcLength()
    xyplot.SetNumberOfXLabels(6)
    xyplot.SetTitle("Var Values Along Arclength")
    xyplot.SetXTitle("")
    xyplot.SetYTitle("")
    xyplot.PlotPointsOn()
    xyplot.PlotLinesOff()
    xyplot.GetProperty().SetColor(0, 0, 0)
    xyplot.GetProperty().SetLineWidth(2)
##     xyplot.SetGlyphSize(2.0)
    tprop = xyplot.GetTitleTextProperty()
    tprop.SetColor(xyplot.GetProperty().GetColor())
    xyplot.SetAxisTitleTextProperty(tprop)
    xyplot.SetAxisLabelTextProperty(tprop)

    xyplot.AddInput(probe.GetOutput())

##     print str(xyplot.GetGlyphSize())

        
    return xyplot

def bigGrey(xmax, xmin, ymax, ymin):
    Xc=(xmax+xmin)/2.0
    Yc=(ymax+ymin)/2.0
    plane = vtk.vtkPlaneSource()
    planeMapper = vtk.vtkPolyDataMapper()
    planeMapper.SetInput(plane.GetOutput())
    planeActor = vtk.vtkActor()
    planeActor.SetMapper(planeMapper)
    planeActor.GetProperty().SetColor(0.6, 0.6, 0.6)
    planeActor.SetPosition(Xc, Yc, -0.001)
    planeActor.SetScale(xmax-xmin, ymax-ymin, 1.0)

    return planeActor
