#!/usr/bin/python

# import python modules
from Tkinter import *
from dialogs import *
import os, string
from configure import *

# the file dialog class
class files:
  def __init__(self, master = None):
    self.toplevel = Toplevel(master, config['frame'])
    self.toplevel.geometry('400x300+200+150')
    self.bottomFrame = Frame(self.toplevel, config['frame'])
    self.bottomFrame.pack(side=BOTTOM, pady=5 )
    self.accept_button = Button(self.bottomFrame, config['button'], 
                                width=7, 
                                text="accept", 
                                command = self.accept)
    self.accept_button.pack(side=LEFT, expand=YES, padx=8)
    self.cancel_button = Button(self.bottomFrame, config['button'], 
                                width=7, 
                                text="cancel", 
                                command = self.cancel)
    self.cancel_button.pack(side=RIGHT, expand=YES, padx=8)
    self.selectionFrame = Frame(self.toplevel, config['frame'])
    self.selectionFrame.pack(side=BOTTOM, padx=36, pady=5, fill=X)
    self.selection = Entry(self.selectionFrame, config['text'], background = 'white')
    self.selection.pack(side=LEFT, expand=YES, fill=X)
    self.directoryFrame = Frame(self.toplevel, config['frame'])
    self.directoryFrame.pack(side=TOP, pady=8)
    self.directory = self.createMenu(self.directoryFrame)
    self.directory.pack(side=LEFT, expand=YES, fill=X)
    self.labelFrame = Frame(self.toplevel, config['frame'])
    self.labelFrame.pack(side=TOP, padx=5, fill=X)
    self.folderLabel = Label(self.labelFrame, config['label'], text = 'folders:')
    self.folderLabel.pack(side=LEFT, expand=YES, fill=X)
    self.fileLabel = Label(self.labelFrame, config['label'], text = ' files: ')
    self.fileLabel.pack(side=RIGHT, expand=YES, fill=X)
    self.middleFrame = Frame(self.toplevel, config['frame'])
    self.middleFrame.pack(expand=YES, padx=5, fill=BOTH)
    self.filesBar = Scrollbar(self.middleFrame, config['scroll'])
    self.filesBar.pack(side=RIGHT, fill=Y)
    self.files = Listbox(self.middleFrame, config['text'], 
                         background = 'white',
                         exportselection=0,
                         yscrollcommand=(self.filesBar, 'set'))
    self.files.pack(side=RIGHT, expand=YES, fill=BOTH)
    btags = self.files.bindtags()
    self.files.bindtags(btags[1:] + btags[:1])
    self.files.bind('<ButtonRelease-1>', self.files_select_event)
    self.files.bind('<Double-ButtonRelease-1>', self.files_double_event)
    self.filesBar.configure(command = self.files.yview)
    self.subBar = Scrollbar(self.middleFrame, config['scroll'])
    self.subBar.pack(side=LEFT, fill=Y)
    self.subDirectory = Listbox(self.middleFrame, config['text'], 
                                background = 'white',
                                exportselection=0,
                                yscrollcommand=(self.subBar, 'set'))
    self.subDirectory.pack(side=LEFT, expand=YES, fill=BOTH)
    self.subBar.configure(command = self.subDirectory.yview)
    btags = self.subDirectory.bindtags()
    self.subDirectory.bindtags(btags[1:] + btags[:1])
    self.subDirectory.bind('<ButtonRelease-1>', self.subDirectory_select_event)

# add the directory menu control to the file dialogue
  def createMenu(self, master = None):
    self.button = Menubutton(master, config['button'], indicatoron = 1)
    self.button.pack()
    self.menu = Menu(self.button, config['menu'], tearoff = 0)
    self.button.configure(menu = self.menu)
    return self.button

# refresh the display when there are user initiated selections
  def refreshDisplay(self, pathString):
    self.myDirectory = pathString
    myList = self.parseDirectory(self.myDirectory)
    self.button.configure(text = myList[0])
    self.menu.delete(0, END)
    for i in myList[1:]:
      self.menu.add_command(label = i, command = self.evaluateDirectory)
    try:
      names = os.listdir(self.myDirectory)
    except os.error:
      self.toplevel.bell()
      return
    names.sort()
    if (self.myDirectory == os.sep) or (not os.sep in self.myDirectory):
      subdirs = []
    else:
      subdirs = [os.pardir]
    matchingfiles = []
    for name in names:
      fullname = os.path.join(self.myDirectory, name)
      if os.path.isdir(fullname) and name[0] != '.':
        subdirs.append(name)
      elif name[0] != '.':
        matchingfiles.append(name)
    self.subDirectory.delete(0, END)
    for name in subdirs:
      self.subDirectory.insert(END, name)
    self.files.delete(0, END)
    for name in matchingfiles:
      self.files.insert(END, name)
    self.set_selection('')

# parse the directory for the directory menu
  def parseDirectory(self, directoryString):
    if directoryString == os.sep:
      return [directoryString]
    if not os.sep in directoryString:
      return [directoryString]
    myList = []
    myString = ''
    myElements = string.split(directoryString, os.sep)
    for element in myElements:
      myString = myString + element
      if element == '':
        myString = os.path.normpath(myString + os.sep)
        myPost = ''
      else:
        myPost = os.sep
      myList.append(myString)
      myString = myString + myPost
    myList.reverse()
    return myList

# process the current directory string  
  def evaluateDirectory(self):
    self.refreshDisplay(self.menu.entrycget(ACTIVE, 'label'))

# set the file selection value
  def set_selection(self, file):
    self.selection.delete('0', END)
    self.selection.insert('0', file)

# get the file selection value
  def get_selection(self):
    return self.selection.get()

# on clicking the subdirectory list
  def subDirectory_select_event(self, event):
    subdir = self.subDirectory.get('active')
    self.refreshDisplay(os.path.normpath(os.path.join(self.myDirectory, subdir)))

# on clicking the file list
  def files_select_event(self, event):
    self.set_selection(self.files.get('active'))

# on double clicking the file list
  def files_double_event(self, event):
    self.accept()

# start the dialog
  def go(self, title = 'file dialog', startPath = os.getcwd()):
    self.newValue = None
    self.toplevel.title(title)
    if os.path.isfile(startPath):
      startPath, myFile = os.path.split(startPath)
      self.refreshDisplay(startPath)
      self.set_selection(myFile)
    else:
      self.refreshDisplay(startPath)
    self.toplevel.grab_set()
    self.toplevel.focus_set()
    self.toplevel.wait_window()
    return self.newValue

# on clicking the accept button
  def accept(self):
    self.newValue = self.get_selection()
    self.toplevel.destroy()

# on clicking the cancel button
  def cancel(self):
    self.toplevel.destroy()

# open files dialog
class openDialog(files):
  def accept(self):
    self.myFile = os.path.join(self.myDirectory, self.get_selection())
    self.newValue = None
    if os.path.isfile(self.myFile):
      self.newValue = self.myFile
      self.toplevel.destroy()
    else:
      message(title = 'error',
              message =  self.myFile + ' is not a file',
              geometry = '250x70+275+265')

# save files dialog
class saveDialog(files):
  def accept(self):
    self.myFile = os.path.join(self.myDirectory, self.get_selection())
    self.newValue = None
    if os.path.isdir(self.myFile):
      message(title = self.myFile,
              message = 'Please enter a file name.',
              geometry = '250x70+275+265')
    elif os.path.isfile(self.myFile):
        self.dialog = question()
        self.answer = self.dialog.go(
                      title = self.myFile,
                      message = 'This file exists!\nOverwrite it?')
        if self.answer:
          self.newValue = self.myFile
          self.toplevel.destroy()
    else:
        self.newValue = self.myFile
        self.toplevel.destroy()

# the test command
if __name__ == '__main__':
  root = Tk()
  root.withdraw()
  myTest1 = openDialog(root)
  print myTest1.go(title = 'open file dialog')
  myTest2 = saveDialog(root)
  print myTest2.go(title = 'save file dialog')
