#!/usr/bin/env python

##############################################################################
##                                                                          ##
##                                                                          ##
##                                                                          ##
##                   #####    ##     ##      ######     ##                  ##
##                 ##   ##   ###    ###     ##   ##    ##                   ##
##                ##        ## ##  ####    #######    ##                    ##
##               ##   ##   ##  ##### ##   ##  ##     ##                     ##
##               #####    ##    ###  ##  ##   ##    #######                 ##
##                                                                          ##
##                                                                          ##
##                      Ohio Supercomputer Center                           ##
##                                                                          ##
##                              08/23/06                                    ##
##                                                                          ##
##                                                                          ##
##                          Dr. Somnath Ghosh                               ##
##                                and                                       ##
##                              Hu Chao                                     ##
##                                                                          ##
##                  Department of Mechanical Engineering                    ##
##                       The Ohio State University                          ##
##                                                                          ##
##                                                                          ##
##                                                                          ##
##   Purpose:                                                               ##
##        The main routine sets up the root and toplevel windows. Creates   ##
##        the widgets for controlling the selection widgets and line        ##
##        widget.                                                           ##
##                                                                          ##
##                                                                          ##
##   Functions:                                                             ##
##         newIncPolyFormat():                                              ##
##              Uses the delauney triangulation to create polygons in       ##
##              the inclusion region.                                       ##
##                                                                          ##
##         newIncTubeFormat():                                              ##
##              Creates a tubular representation of the hole in             ##
##              the inclusion region.                                       ##
##                                                                          ##
##         newMatFormat():                                                  ##
##              Uses the delauney triangulation to create polygons in       ##
##              the matrix region. Responsible for keeping track of the     ##
##              inclusion regions.                                          ##
##                                                                          ##
##                                                                          ##
##                                                                          ##
##                                                                          ##
##############################################################################

import os
import string 
import vtk
from vtk.util.colors import *
global xmax,xmin,ymax,ymin
global maxval
global minval
##maxval = 0.0
##minval = 0.0
def newIncPolyFormat(dictVals, varkey, file,Icolor,maxval,minval):
#    global maxval
#    global minval
    revmaxval = 0.0
    varptr = 4
    incident = 0
    numpts = 0
    numedgepts = 0
    numholes = 0
    numholepts = 0
    numholes = 0
    numholepts = 0
    holeptindx = 0
    curhole = 0


    lut=vtk.vtkLookupTable()
    if (Icolor==1):
    	lut.SetHueRange(0.66667,0)
    else:
	lut.SetHueRange(0,0)
        lut.SetSaturationRange(0,0)
        lut.SetValueRange(.75, 0.25)


    points = vtk.vtkPoints()
    varval = vtk.vtkFloatArray()
    polys = vtk.vtkCellArray()
    polyData = vtk.vtkPolyData()
    points.Initialize()
    varval.Initialize()
    varval.SetName(varkey)
    polys.Initialize()
    polyData.Initialize()

    holepoints = vtk.vtkPoints()
    holepolys = vtk.vtkCellArray()
    holepolyData = vtk.vtkPolyData()
    holepoints.Initialize()
    holepolys.Initialize()
    holepolyData.Initialize()

    novar=0
    if (dictVals.has_key(varkey)):
##        if maxval==0 and minval==0:
##           maxval = float(dictVals[varkey][1])
##           minval = float(dictVals[varkey][0])
        
        revmaxval = maxval - minval
        varptr = int(varkey[-1]) + 3
    else:
        novar=1
    # This line should read INC int, the identity of the inclusion 
    line = file.readline()
    data = string.split(line)
    #print "Inside Inc Poly:: " + line
    if data[1] == 'INC':
        incident = int(data[2])
    
    line = file.readline()
    data = string.split(line)
    #print "Inside Inc Poly:: " + line
    if data[1] == 'NUMPTS':
        numpts = int(data[2])
    
    line = file.readline()
    data = string.split(line)
    #print "Inside Inc Poly:: " + line
    if data[1] == 'NUMEDGE':
        numedgepts = int(data[2])
        polys.InsertNextCell(numedgepts)
    
    line = file.readline()
    data = string.split(line)
    #print "Inside Inc Poly:: " + line
    if data[1] == 'NUMHOLES':
        numholes = int(data[2])

    i = 0
    while i < numpts:
        line = file.readline()
        data = string.split(line)

	if novar == 1:
           data[varptr]=0.0
        if data[0] == 'e':
            index, x, y, scalval = int(data[1]), float(data[2]), \
                                   float(data[3]), float(data[varptr])
            points.InsertPoint(index, x, y, 0.0)
            polys.InsertCellPoint(index)
            varval.InsertNextValue(scalval)
            i = i + 1
        elif data[0] == 'i':
            index, x, y, scalval = int(data[1]), float(data[2]), \
                                   float(data[3]), float(data[varptr])
            points.InsertPoint(index, x, y, 0.0)
            varval.InsertNextValue(scalval)
            i = i + 1
        elif data[0] == 'h':
            index, x, y, scalval = int(data[1]), float(data[2]), \
                                   float(data[3]), float(data[varptr])
            points.InsertPoint(index, x, y, 0.0)
            polys.InsertCellPoint(index)
            varval.InsertNextValue(scalval)
            i = i + 1
            
            holepoints.InsertPoint(holeptindx, x, y, 0.0)
            holeptindx = holeptindx + 1

        elif data[0] == '#' and data[1][:4] == 'HOLE':
            curhole = int(data[1][4:])
            numholepts = int(data[2])
            polys.InsertNextCell(numholepts)
        else:
            print "OOPS again "

    polyData = vtk.vtkPolyData()
    polyData.SetPoints(points)
    polyData.SetPolys(polys)
    polyData.GetPointData().SetScalars(varval)

    delny = vtk.vtkDelaunay2D()
    delny.SetInput(polyData)
    delny.SetSource(polyData)
            
    extract = vtk.vtkExtractEdges()
    extract.SetInputConnection(delny.GetOutputPort())
    tubes = vtk.vtkTubeFilter()
    tubes.SetInputConnection(extract.GetOutputPort())
    tubes.SetRadius(0.001)
    tubes.SetNumberOfSides(6)
    mapEdges = vtk.vtkPolyDataMapper()
    mapEdges.SetInputConnection(tubes.GetOutputPort())
    edgeActor = vtk.vtkActor()
    edgeActor.SetMapper(mapEdges)
    edgeActor.GetProperty().SetColor(peacock)
    edgeActor.GetProperty().SetSpecularColor(1, 1, 1)
    edgeActor.GetProperty().SetSpecular(0.3)
    edgeActor.GetProperty().SetSpecularPower(20)
    edgeActor.GetProperty().SetAmbient(0.2)
    edgeActor.GetProperty().SetDiffuse(0.8)	

    mapMesh = vtk.vtkPolyDataMapper()
    mapMesh.SetInputConnection(delny.GetOutputPort())
    mapMesh.SetLookupTable(lut)
    mapMesh.SetScalarRange(minval, maxval)

    meshActor = vtk.vtkActor()
    meshActor.SetMapper(mapMesh)

    if numholes > 0:
        holepolyData = vtk.vtkPolyData()
        holepolyData.SetPoints(holepoints)
        holepolyData.SetPolys(holepolys)
        
        holeDelny = vtk.vtkDelaunay2D()
        holeDelny.SetInput(holepolyData)
        holeDelny.SetSource(holepolyData)
        holeMapMesh = vtk.vtkPolyDataMapper()
        holeMapMesh.SetInput(holeDelny.GetOutput())
        spackleActor = vtk.vtkActor()
        spackleActor.SetMapper(holeMapMesh)
        spackleActor.GetProperty().SetColor(white)
        success = 1
    else:
        spackleActor = vtk.vtkActor()
        success = 0
    
    return success, meshActor, delny, spackleActor

def newIncTubeFormat(dictVals, varkey, file,Icolor,maxval,minval):
#    global maxval
#    global minval
    varptr = 4
#    maxval = 0.0
#    minval = 0.0
    revmaxval = 0.0
    varptr = 4
    incident = 0
    numpts = 0
    numedgepts = 0
    numholes = 0
    numholepts = 0
    holeptindx = 0
    curhole = 0
    success = 0

    points = vtk.vtkPoints()
    polys = vtk.vtkCellArray()
    polyData = vtk.vtkPolyData()
    points.Initialize()
    polys.Initialize()
    polyData.Initialize()

    holepoints = vtk.vtkPoints()
    holepolys = vtk.vtkCellArray()
    holepolyData = vtk.vtkPolyData()
    holepoints.Initialize()
    holepolys.Initialize()
    holepolyData.Initialize()

    if (dictVals.has_key(varkey)):
        #print "We have an inclusion variable called " + varkey + " = (" + \
        #      dictVals[varkey][0] + ", " + dictVals[varkey][1] + ")"
##        if maxval==0 and minval==0:
##           maxval = float(dictVals[varkey][1])
##           minval = float(dictVals[varkey][0])
        #revmaxval = maxval - minval
        #print "revmaxval is " + str(revmaxval)
        varptr = int(varkey[-1]) + 3

    # This line should read INC int, the identity of the inclusion 
    line = file.readline()
    data = string.split(line)
    if data[1] == 'INC':
        incident = int(data[2])
        ## print "INC number " + str(incident)
    
    line = file.readline()
    data = string.split(line)
    if data[1] == 'NUMPTS':
        numpts = int(data[2])
        ## print "Number points " + str(numpts)
    
    line = file.readline()
    data = string.split(line)
    if data[1] == 'NUMEDGE':
        numedgepts = int(data[2])
        #    polys.InsertNextCell(numedgepts)
        ## print "Number of edge pts " + str(numedgepts)
    
    line = file.readline()
    data = string.split(line)
    if data[1] == 'NUMHOLES':
        numholes = int(data[2])

    i = 0
    while i < numpts:
        line = file.readline()
        data = string.split(line)
        if data[0] == 'e':
            index, x, y, scalval = int(data[1]), float(data[2]), \
                                   float(data[3]), float(data[varptr])
            points.InsertPoint(index, x, y, 0.0)
            #   polys.InsertCellPoint(index)
            #scalval = maxval - scalval
            #varval.InsertNextValue(scalval)
            i = i + 1
        elif data[0] == 'i':
            index, x, y, scalval = int(data[1]), float(data[2]), \
                                   float(data[3]), float(data[varptr])
            points.InsertPoint(index, x, y, 0.0)
            #scalval = maxval - scalval
            #varval.InsertNextValue(scalval)
            i = i + 1
        elif data[0] == 'h':
            index, x, y, scalval = int(data[1]), float(data[2]), \
                                   float(data[3]), float(data[varptr])
            points.InsertPoint(index, x, y, 0.0)
            polys.InsertCellPoint(index)
            #scalval = maxval - scalval
            #varval.InsertNextValue(scalval)
            i = i + 1
            
            holepoints.InsertPoint(holeptindx, x, y, 0.0)
            holeptindx = holeptindx + 1
            
        elif data[0] == '#' and data[1][:4] == 'HOLE':
            curhole = data[1][4:]
            ## print "Found hole number " + str(curhole)
            numholepts = int(data[2])
            polys.InsertNextCell(numholepts)
            
        else:
            print "OOPS again: " + str(i) + " : " + line
        

    if (numholes > 0):
        polyData = vtk.vtkPolyData()
        polyData.SetPoints(points)
        polyData.SetPolys(polys)
        
        holepolyData = vtk.vtkPolyData()
        holepolyData.SetPoints(holepoints)
        holepolyData.SetPolys(holepolys)

        extract = vtk.vtkExtractEdges()
        extract.SetInput(polyData)
        tubes = vtk.vtkTubeFilter()
        tubes.SetInputConnection(extract.GetOutputPort())
        tubes.SetRadius(0.002)
        tubes.SetNumberOfSides(6)
        mapEdges = vtk.vtkPolyDataMapper()
        mapEdges.SetInputConnection(tubes.GetOutputPort())
        edgeActor = vtk.vtkActor()
        edgeActor.SetMapper(mapEdges)
        edgeActor.GetProperty().SetColor(black)
        edgeActor.GetProperty().SetSpecularColor(1, 1, 1)
        edgeActor.GetProperty().SetSpecular(0.3)
        edgeActor.GetProperty().SetSpecularPower(20)
        edgeActor.GetProperty().SetAmbient(0.2)
        edgeActor.GetProperty().SetDiffuse(0.8)
        
        delny = vtk.vtkDelaunay2D()
        delny.SetInput(holepolyData)
        delny.SetSource(holepolyData)
        mapMesh = vtk.vtkPolyDataMapper()
        mapMesh.SetInput(delny.GetOutput())
        spackleActor = vtk.vtkActor()
        spackleActor.SetMapper(mapMesh)
        spackleActor.GetProperty().SetColor(white)

        success = 1
    else:
        edgeActor = vtk.vtkActor()
        spackleActor = vtk.vtkActor()
        
        success = 0
    
    return success, edgeActor, spackleActor

def newMatFormat(dictVals, varkey, file,Icolor,maxval,minval):
#    global maxval
#    global minval
    global xmax,xmin,ymax,ymin

    varptr = 4
#    maxval = 0.0
#    minval = 0.0
    xmax = -1.E+20
    xmin = 1.E+20
    ymax = -1.E+20
    ymin = 1.E+20
    revmaxval = 0.0
    fini = 1
	
    lut=vtk.vtkLookupTable()
    if (Icolor==1):
    	lut.SetHueRange(0.66667,0)
    else:
	lut.SetHueRange(0,0)
        lut.SetSaturationRange(0,0)
        lut.SetValueRange(.75, 0.25)

    points = vtk.vtkPoints()
    varval = vtk.vtkFloatArray()
    polys = vtk.vtkCellArray()
    polyData = vtk.vtkPolyData()

    points.Initialize()
    varval.Initialize()
    varval.SetName(varkey)
    polys.Initialize()
    polyData.Initialize()

    ## print "Matrix var key " + varkey
    if (dictVals.has_key(varkey)):
        ## print "We have a matrix variable called " + varkey + " = (" + \
        ##      dictVals[varkey][0] + ", " + dictVals[varkey][1] + ")"
##        maxval = float(dictVals[varkey][1])
        #maxval =0.03
##        minval = float(dictVals[varkey][0])
        #minval =0.1
        revmaxval = maxval - minval
        ## print "revmaxval is " + str(revmaxval)
        varptr = int(varkey[-1]) + 3
	## print "varptr is " + str(varptr)
    while (fini > 0):
        line = file.readline()
        data = string.split(line)
        #print "Inside mat " + line
        
        if data and data[0] == '#':
            if data[1] == 'NELX':
                numincl = int(data[2])
                numtotpts = int(data[3])
            elif data[1] == 'EDGELIST':
                numedgepts = int(data[2])
                polys.InsertNextCell(numedgepts)
                i = 0
                while (i < numedgepts):
                    line = file.readline()
                    data = string.split(line)
	            #print "Inside mat " + line
                    if data and data[0] == 'e':
	                
                        index, x, y, scalval = \
                               int(data[1]), float(data[2]), \
                               float(data[3]), float(data[varptr])
                        #print "index= " +str(index)
	                if (x > xmax):
                           xmax = x
                        if (x < xmin):
                           xmin = x
                        if (y > ymax):
                           ymax = y
                        if (y < ymin):
                           ymin = y
                        points.InsertPoint(index, x, y, 0.0)
                        polys.InsertCellPoint(index)
                        #scalval = maxval - scalval
                        varval.InsertNextValue(scalval)
                        i = i + 1
                    else:
                        print "OOPS in Matrix edge listing"
            elif data[1] == 'PTLIST':
                numpts = int(data[2])
                i = 0
                while (i < numpts):
                    line = file.readline()
                    data = string.split(line)
                    # print "PTLIST " + line
                    
                    if data and data[0] == 'm':
                        index, x, y, scalval = \
                               int(data[1]), float(data[2]), \
                               float(data[3]), float(data[varptr])
                        points.InsertPoint(index, x, y, 0.0)
                        #scalval = maxval - scalval
                        varval.InsertNextValue(scalval)
                        i = i + 1
                    else:
                        print "OOPS in Matrix general listing"
            elif data[1] == 'Hole':
                curhole = int(data[2])
                numholepts = int(data[3])
                polys.InsertNextCell(numholepts)
                i = 0
                while (i < numholepts):
                    line = file.readline()
                    data = string.split(line)
                    if data and data[0] == 'h':
                        index, x, y, scalval = \
                               int(data[1]), float(data[2]), \
                               float(data[3]), float(data[varptr])
                        points.InsertPoint(index, x, y, 0.0)
                        polys.InsertCellPoint(index)
                        #scalval = maxval - scalval
                        varval.InsertNextValue(scalval)
                        i = i + 1
                    else:
                        print "OOPS in Matrix hole " + str(curhole)
            elif data[1] == 'ENDIST':
                fini = -1


    polyData = vtk.vtkPolyData()
    polyData.SetPoints(points)
    polyData.SetPolys(polys)
    polyData.GetPointData().SetScalars(varval)
    
    delny = vtk.vtkDelaunay2D()
    delny.SetInput(polyData)
    delny.SetSource(polyData)


    extract = vtk.vtkExtractEdges()
    extract.SetInputConnection(delny.GetOutputPort())
    tubes = vtk.vtkTubeFilter()
    tubes.SetInputConnection(extract.GetOutputPort())
    tubes.SetRadius(0.001)
    tubes.SetNumberOfSides(6)
    mapEdges = vtk.vtkPolyDataMapper()
    mapEdges.SetInputConnection(tubes.GetOutputPort())
    edgeActor = vtk.vtkActor()
    edgeActor.SetMapper(mapEdges)
    edgeActor.GetProperty().SetColor(peacock)
    edgeActor.GetProperty().SetSpecularColor(1, 1, 1)
    edgeActor.GetProperty().SetSpecular(0.3)
    edgeActor.GetProperty().SetSpecularPower(20)
    edgeActor.GetProperty().SetAmbient(0.2)
    edgeActor.GetProperty().SetDiffuse(0.8)


    mapMesh = vtk.vtkPolyDataMapper()
    mapMesh.SetInputConnection(delny.GetOutputPort())
    mapMesh.SetLookupTable(lut)
    mapMesh.SetScalarRange(minval, maxval)
    	
	
    meshActor = vtk.vtkActor()
    meshActor.SetMapper(mapMesh)

    return meshActor, edgeActor, delny, xmax, xmin, ymax, ymin    


