C------------------------------------------------------------------
C     voronoi cell finite element method mesh generation program
C------------------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z) 
      CALL random1(vf,np)
      STOP
      END
C------------------------------------------------------------

      subroutine random1(vf,np)
C------------------------------------------------------------
C Purpose:
C   generates NE random circles/ellipses such that 
C   - center coords. Xc,Yc - are random within (0,0),(1,1) 
C     square region.  
C
C Req'd suroutines: elcut, elints, laguer, zroots, inellip1
C
C Written by:      Miroslav Dujin
C Latest revision:  19 June 95
C
C-------------------------------------------------------------

      IMPLICIT REAL*8 (A-H,O-Z) 

      parameter (k1=500,k2=2000)

      integer  choice
      real     time,dtime,tarr(2),timeel(k1)
      real     ran
      real*8   xc(k1),yc(k1),a(k1),boa(k1),b(k1),theta(k1),
     *         xc00(k2,k2),yc00(k2,k2),xc0(k1),yc0(k1),
     *         np1,asr,xic(10),yic(10)
      logical  flag,T,F
      common   conv,nran  ! conv.fact. [rad]->[deg]
      common   /eangle/ start_angle,end_angle


c     CPU time initialization:
      time = dtime(tarr)
      

c     Initialization of SEED
      write(*,*)'Enter a random seed number'
      READ(5,*) nran
      nran=10000*nran
      write(*,*)ran(nran)

      T = .true.
      F = .false.
      pi = 4.*atan(1.)
      pihlf = pi/2.
      conv = 180./pi  ! [rad]->[deg]
      
C      enlg = 1.05  ! particulate radius(es) enlargement
C                   ! - to avoid close particulates 
c
      print*
      print*,'Program RANGEN:'
      print*,'---------------'
      print*,'Choose one option - particulate type:'
      print*,'- circles ........ 1'
      print*,'- ellipses ....... 2'
 900  read*,choice
      print*
      if ((choice.ne.1).and.(choice.ne.2)) goto 900

      print*,'Specify volume fraction Vf [%] :'       
      read*,vf
      print*,'Specify domain coordinate(x1,y1,x2,y2) eg.(0,0,1,1) :'
      read*,cx1,cy1,cx2,cy2
      hs=dabs(cx2-cx1)
      vs=dabs(cy2-cy1)
      print*,'No. of particulates to be generated:'
      print*,'IF you want vary paticle size too, type "0"'
      read*,np
      ip=np
      if (ip.eq.0) then
         call real_random(vf,choice,np,cx1,cx2,cy1,cy2)
         return
      endif
C----------------------------------------------------------
      print*,'Input number of cluster (Zero for no clustering):'
      read*,iclus            

c---------------------------------------------------
c calculate circle/ellipse radius(es) including corr.
c due to octagonal approx of contour (Vf preservation):
c---------------------------------------------------
      area = 1.                 ! unit domain area
      area = hs*vs
      if (choice.eq.1) then  ! circle
        oarea = area*vf/100./np ! one circle
        ocr = 1.0  ! octagon/circle area ratio
        oarea = oarea/ocr
        radius = dsqrt(oarea/pi) ! of eqv.circle
        goto 1
      elseif (choice.eq.2) then  ! ellipse
        print*,'Input ellipse aspect ratio a/b:(a>b)'
        read*,asr
        asr=1./asr
        oarea = area*vf/100./np ! one ellipse
c det.eqv.ellipse parameters which will be discretized
c so that polygonal cnt's have prescribed Vf:
        ra = dsqrt(oarea/asr/pi)  ! orig.el.maj.axis
c get octagon/ellip.area ratio
C        call eqvelip(ra,asr,oer)
        oer=1.0 
        oarea1 = oarea/oer        ! new.el.area 
        ra1 = dsqrt(oarea1/asr/pi) ! new.el.axes:
        rb1 = asr*ra1
        print*,'Input ellipse orientation range(start_angle,end_angle):'
        read*,start_angle,end_angle
        start_angle=start_angle*3.141592/180.
        end_angle=end_angle*3.141592/180.
        goto 1  
      endif
c------------------------------------
c     determine clustering position
c------------------------------------
 1    if (iclus.eq.0) goto 18
      print*,'Input variation factor for particle center
     *  eg.(4 times the radius of the particle):'      
      if(choice.eq.1)
     * print*,'radius of particles =',radius
      if(choice.eq.2)
     * print*,'average of radius of particles =',(ra1+rb1)/2.
      read*,rvar
      print*,
     *'Input # of particles at each cluster(same for all clusters):'
      read*,nic
      print*,'Specify minimum permissible distance in clustering :' 
      read*,enlg
      np0=np/2
      do i=1,iclus
         print*,'Input clustering coordinates: x,y',i
         read*,xic(i),yic(i)
      enddo
c      nc=np0/iclus
      ni=1
      do k=1,iclus
         do i=ni,ni+nic-1
            xc0(i)=xic(k)
            yc0(i)=yic(k)
         enddo
         ni=ni+nic
      enddo
      np0=nic*iclus
      i=1
      goto 5
c-------------------------------
c determine regular distribution :
c------------------------------
 18   print*,'Input variation factor for particle center
     *  eg.(4 times the radius of the particle):' 
      if(choice.eq.1)
     * print*,'Radius of particles =',radius
      if(choice.eq.2)
     * print*,'Average of radius of particles =',(ra1+rb1)/2.
      read*,rvar 
      print*,'Specify minimum permissible distance :' 
      read*,enlg
      np1 = np                  ! conv.to real
      nstep = int(sqrt(np1))
      if (nstep*nstep.eq.np) then
        np0 = np   ! # of initial centroid position
        goto 2
      endif
      np0 = nstep*nstep ! # of initial centroid position
      
c---------------------------------------------------------
c locate initial position of uniformly distributed
c particulate centeroids:
c---------------------------------------------------------
 2    dstep = 1./nstep

      do 3  kc = 1,nstep  ! column counter
        do 3 kr = 1,nstep  ! row counter
          xc00(kr,kc) = (dstep/2.+(kc-1)*dstep)*hs
          yc00(kr,kc) = (1.-dstep/2.-(kr-1)*dstep)*vs
 3    continue
c--------------------------------------------
c copy to 1D arrays (only NP0 centroids =< NP):
c---------------------------------------------
      do 4  kr = 1,nstep
        do 4  kc = 1,nstep
          k = (kr-1)*nstep+kc
          xc0(k) = xc00(kr,kc)
          yc0(k) = yc00(kr,kc)
 4    continue
c --------------------------------------------------- 
c generate 1st particle
c----------------------------------------------------
      i = 1

 5    if (choice.eq.1) then     ! circle     
        call gencir (xc0(1),yc0(1),radius,rvar,theta(1),
     *               xc(1),yc(1),hs,vs)
        a(1) = radius
        b(1) = radius
        boa(1) = 1.
      elseif (choice.eq.2) then  ! ellipse
        call genelip (xc0(1),yc0(1),ra1,rb1,rvar,theta(1),
     *                xc(1),yc(1),hs,vs)
        a(1) = ra1
        b(1) = rb1
        boa(1) = asr
      endif 
c-----------------------
c boundary constraint:
c-----------------------
      if (choice.eq.1) then  ! circle
        call chkbndc (cx1+enlg/4.,cy1+enlg/4.,cx2-enlg/4.,
     &        cy2-enlg/4.,xc(1),yc(1),a(1),flag)
        if (flag.eq.F) then   ! intersect with bound.
          goto 5  ! generate new cir.#1
        endif
        goto 6   ! write info
      elseif (choice.eq.2) then  ! ellipse
        call chkbnde (cx1+enlg/4.,cy1+enlg/4.,cx2-enlg/4.,cy2-enlg/4.,
     &        xc(1),yc(1),a(1),
     *        b(1),theta(1),flag)
        if (flag.eq.F) then   ! intersect with bound.
          goto 5  ! generate new el.#1
        endif
        goto 6   ! write info
      endif

 6    print*,'Particulate # 1 generated...' 
      time = dtime(tarr)
      timeel(1) = tarr(1)
      print*,'gen.time=',timeel(1),' [sec]'
      tsum = timeel(1)
      print*
c------------------------------------
c Now generate all other particulates:
c------------------------------------
 10   if(np.lt.4) goto 13
      i = i+1  ! curr.particulate

 11   if (choice.eq.1) then     ! circle     
        call gencir (xc0(i),yc0(i),radius,rvar,theta(i),
     *               xc(i),yc(i),hs,vs)
        a(i) = radius
        b(i) = radius
        boa(i) = 1.
      elseif (choice.eq.2) then  ! ellipse
        call genelip (xc0(i),yc0(i),ra1,rb1,rvar,theta(i),
     *                xc(i),yc(i),hs,vs)
        a(i) = ra1
        b(i) = rb1
        boa(i) = asr
      endif
c---------------------
c boundary check
c---------------------
      if (xc(i).le.cx1.or.xc(i).ge.cx2.or.yc(i).le.cy1.or.
     $     yc(i).ge.cy2) goto 11
      if (choice.eq.1) then  ! circle
        call chkbndc (cx1+enlg/4.,cy1+enlg/4.,cx2-enlg/4.,
     &        cy2-enlg/4.,xc(i),yc(i),a(i),flag)
        if (flag.eq.F) then   ! intersect with bound.
          goto 11  ! generate new cir. #i
        endif
        goto 12   ! overlaping chk.
      elseif (choice.eq.2) then  ! ellipse
        call chkbnde (cx1+enlg/4.,cy1+enlg/4.,cx2-enlg/4.,cy2-enlg/4.,
     &        xc(i),yc(i),a(i),
     *        b(i),theta(i),flag)
        if (flag.eq.F) then   ! intersect with bound.
          goto 11  ! generate new el. #i
        endif
        goto 12   !  overlaping chk.
      endif
c-----------------------------
c particle intersecting check
c-----------------------------
 12   do 121  j = 1,i-1          ! loop over all prev.gen.cir.

        if (choice.eq.1) then     ! circle
          call chkolcr (xc(i),yc(i),xc(j),yc(j),
     *                  a(i)+enlg,a(j),flag)
          if (flag.eq.T) goto 11  ! generate again cir. #i
        elseif (choice.eq.2) then ! ellipse
          call chkolep (xc(i),yc(i),a(i)+enlg,b(i)+enlg,
     *                  theta(i),xc(j),yc(j),a(j),
     *                  b(j),theta(j),flag)
          if (flag.eq.T) goto 11  ! generate again el. #i
        endif

 121  continue

      print*,'Particulate #',i,' generated...'
      time = dtime(tarr)
      timeel(i) = tarr(1)
      print*,'gen.time=',timeel(i),' [sec]'
      tsum = tsum+timeel(i)
      print*

      if (i .eq. np0) then  ! all part.generated ?
        goto 13  ! gen. surplus particles
      else
        goto 10  ! next I
      endif
c------------------------------------------------------------
c Now place rest of the particles (NP-NP0) in aviliable space
c using random centroid location:
c-----------------------------------------------------------
 13   ndif = np-np0
      if (ndif.eq.0) goto 20  ! end of generation
      if(iclus.gt.0) then
         print*,'Min. permissible dist. for outside particles?'
         read*, enlg
      endif

 14   i = i+1   ! one more particulate
     
 15   xcran = cx1+ran(nran)*(cx2-cx1)
      ycran = cy1+ran(nran)*(cy2-cy1)

      if (choice.eq.1) then     ! circle     
         xc(i)=xcran 
         yc(i)=ycran 
         if (xc(i).le.cx1.or.xc(i).ge.cx2.or.yc(i).le.cy1.or.
     $       yc(i).ge.cy2) goto 15
         pi = 4.*atan(1.)
         theta(i) = ran(nran)*2.*pi*conv ! [deg]
         a(i) = radius
         b(i) = radius
         boa(i) = 1.
      elseif (choice.eq.2) then  ! ellipse
         xc(i)=xcran 
         yc(i)=ycran 
         pi = 4.*atan(1.)
         theta(i) = (start_angle+ran(nran)*(end_angle-start_angle))*conv
         a(i) = ra1
         b(i) = rb1
         boa(i) = asr
      endif  
c-------------------------
c boundary constraint:
c-------------------------
      if (choice.eq.1) then  ! circle
        call chkbndc (cx1+enlg/4.,cy1+enlg/4.,cx2-enlg/4.,
     &        cy2-enlg/4.,xc(i),yc(i),a(i),flag)
        if (flag.eq.F) then   ! intersect with bound.
          goto 15  ! generate again cir. #i
        endif
        goto 16   ! overlaping chk.
      elseif (choice.eq.2) then  ! ellipse
        call chkbnde (cx1+enlg/4.,cy1+enlg/4.,cx2-enlg/4.,cy2-enlg/4.,
     &        xc(i),yc(i),a(i),
     *        b(i),theta(i),flag)
        if (flag.eq.F) then   ! intersect with bound.
          goto 15  ! generate again el. #i
        endif
        goto 16   !  overlaping chk.
      endif
c----------------------------------------------
c disable intersection of particulate contours :
c----------------------------------------
 16   do 17  j = 1,i-1           ! loop over all prev.gen.cir.

        if (choice.eq.1) then     ! circle
          call chkolcr (xc(i),yc(i),xc(j),yc(j),a(i)+enlg,
     *                  a(j),flag)
          if (flag.eq.T) goto 15  ! generate again cir. #i
          goto 17                 ! check next cir.J
        elseif (choice.eq.2) then ! ellipse
          call chkolep (xc(i),yc(i),a(i)+enlg,b(i)+enlg,
     *                  theta(i),xc(j),yc(j),a(j),
     *                  b(j),theta(j),flag)
          if (flag.eq.T) goto 15  ! generate again el. #i
          goto 17                 ! check next el.J      
        endif

 17   continue

      print*,'Particulate #',i,' generated...'
      time = dtime(tarr)
      timeel(i) = tarr(1)
      print*,'gen.time=',timeel(i),' [sec]'
      tsum = tsum+timeel(i)
      print*

 20   if (i.eq.np) then
        call outPS (cx1,cy1,cx2,cy2,np,xc,yc,a,boa,b,theta,hs,vs)
        goto 21   ! goto end
      else
        goto 14    ! next 'i'
      endif
c----------------------------------------------------------
 21   time = dtime(tarr)
      write (*,*)
      write (*,*) 'CPU time [sec] =', tsum+tarr(1)
      write (*,*) 'tsum [sec] =', tsum

      return 
      end

C----------------------------------------------------------
      subroutine real_random(vf,choice,np,cx1,cx2,cy1,cy2)
C----------------------------------------------------------
C      modify program to make all the particle distribution 
C      random 
C----------------------------------------------------------
      IMPLICIT REAL*8 (A-H,O-Z) 

      parameter (k1=500,k2=2000)
      integer  choice
      real ran
      real*8   mpdis
      real     time,dtime,tarr(2),timeel(k1)
      real*8   xc(k1),yc(k1),a(k1),boa(k1),b(k1),theta(k1),
     *         xc00(k2,k2),yc00(k2,k2),xc0(k1),yc0(k1),
     *         np1,asr,xic(10),yic(10),area(k1)
      logical  flag,T,F
      common   conv,nran  ! conv.fact. [rad]->[deg]
      T = .true.
      F = .false.
      hs=dabs(cx2-cx1)
      vs=dabs(cy2-cy1)
C----------------------------------------------------------
      print*,'Please specify range of particle size:'
      print*,'min and max w.r.t the total area of the domain'
      print*,'give percents(for ex. 0.5,2)'
      read*,amin,amax
      amin=amin/100.
      amax=amax/100.
      if(choice.eq.2) then
         print*,'I need range of a/b ratio for ellipse'
         print*,'give me maximun valuse, minimum is 1.'
         read*, boamax
      endif
c---------------------------------------------------
c     calculate mean radius 
c---------------------------------------------------
      pi = 4.*atan(1.)
      amean=(amin+amax)/2.0
      rmean=dsqrt(amean/pi)       

      print*,'Do you need clustering? input number of cluster:'
      read*,iclus            
      do i=1,iclus
         print*,'input clustering coordinates :#',i
         read*,xic(i),yic(i)
      enddo
      print*,'Specify minimum permissible distance :' 
      read*,enlg
c --------------------------------------------------- 
c generate 1st particle
c----------------------------------------------------
      i = 1
 5    if (choice.eq.1) then     ! circle     
         xc(1) = cx1+ran(nran)*(cx2-cx1)
         yc(1) = cy1+ran(nran)*(cy2-cy1)
         area(1)=amin+ran(nran)*(amax-amin)
         a(1) = dsqrt(area(1)/pi)
         b(1) = a(1)
         boa(1) = 1.
      elseif (choice.eq.2) then  ! ellipse
         cdist = ran(nran)*rmean*rvar
         xc(1) = cx1+ran(nran)*(cx2-cx1)
         yc(1) = cy1+ran(nran)*(cy2-cy1)
         theta(1) = ran(nran)*360.      
         area(1)=amin+ran(nran)*(amax-amin)
         boa(1) = 1.+(boamax-1.0)*ran(nran)
         b(1) = dsqrt(area(1)/boa(1)/pi)
         a(1) = boa(1)*b(1)
      endif 
c-----------------------
c boundary constraint:
c-----------------------
      if (xc(1).le.cx1.or.xc(1).ge.cx2.or.yc(1).le.cy1.or.
     $     yc(1).ge.cy2) goto 5
      if (choice.eq.1) then  ! circle
        call chkbndc (cx1+enlg/4.,cy1+enlg/4.,cx2-enlg/4.,
     &        cy2-enlg/4.,xc(1),yc(1),a(1),flag)
         if (flag.eq.F) then    ! intersect with bound.
            goto 5              ! generate new cir.#1
         endif
        goto 6   ! write info
      elseif (choice.eq.2) then  ! ellipse
        call chkbnde (cx1+enlg/4.,cy1+enlg/4.,cx2-enlg/4.,cy2-enlg/4.,
     &        xc(1),yc(1),a(1),
     *        b(1),theta(1),flag)
        if (flag.eq.F) then   ! intersect with bound.
          goto 5  ! generate new el.#1
        endif
        goto 6   ! write info
      endif

 6    total_area=area(1)
      print*,'Particulate # 1 generated...' 
      time = dtime(tarr)
      timeel(1) = tarr(1)
      print*,'gen.time=',timeel(1),' [sec]'
      tsum = timeel(1)
      print*
c------------------------------------
c Now generate all other particulates:
c------------------------------------
 10   i = i+1                   ! curr.particulate

 11   if (choice.eq.1) then     ! circle     
         xc(i) = cx1+ran(nran)*(cx2-cx1)
         yc(i) = cy1+ran(nran)*(cy2-cy1)
         area(i)=amin+ran(nran)*(amax-amin)
         ra=vf*hs*vs/100.-total_area
         if(ra.lt.(amax+amin)) then
            if(ra.gt.2.*amin) then
               area(i)=amin
            else
               area(i)=ra
            endif
         endif
         b(i) = dsqrt(area(i)/pi)
         a(i) = b(i)
         boa(i) = 1.
      elseif (choice.eq.2) then  ! ellipse
         xc(i) = cx1+ran(nran)*(cx2-cx1)
         yc(i) = cy1+ran(nran)*(cy2-cy1)
         theta(i) = ran(nran)*360.
         area(i)=amin+ran(nran)*(amax-amin)
         ra=vf/100.*hs*vs-total_area
         if(ra.lt.(amax+amin)) then
            if(ra.gt.amax) then
               area(i)=amin
            else
               area(i)=ra
            endif
         endif
         boa(i) = 1.+(boamax-1.0)*ran(nran)
         b(i) = dsqrt(area(i)/boa(i)/pi)
         a(i) = boa(i)*b(i)
      endif 

c---------------------
c boundary check
c---------------------
      if (choice.eq.1) then  
         call chkbndc (cx1+enlg/4.,cy1+enlg/4.,cx2-enlg/4.,
     &        cy2-enlg/4.,xc(i),yc(i),a(i),flag)
         if (flag.eq.F) goto 11 
      elseif (choice.eq.2) then  
        call chkbnde (cx1+enlg/4.,cy1+enlg/4.,cx2-enlg/4.,cy2-enlg/4.,
     &        xc(i),yc(i),a(i),
     *        b(i),theta(i),flag)
         if (flag.eq.F) goto 11
      endif
c-----------------------------
c particle intersecting check
c-----------------------------
 12   do 121  j = 1,i-1          ! loop over all prev.gen.cir.     
         if (choice.eq.1) then  
            call chkolcr (xc(i),yc(i),xc(j),yc(j),
     *           a(i)+enlg,a(j),flag)
            if (flag.eq.T) goto 11 
         elseif (choice.eq.2) then ! ellipse
            call chkolep (xc(i),yc(i),a(i),b(i),
     *           theta(i),xc(j),yc(j),a(j)+enlg,
     *           b(j)+enlg,theta(j),flag)
            if (flag.eq.T) goto 11 ! generate again el. #i
         endif    
 121  continue

      total_area=total_area+area(i)

      print*,'Particulate #',i,' generated...'
      print*,'Particle Volume = ',total_area,' generated...'
      time = dtime(tarr)
      timeel(i) = tarr(1)
      print*,'gen.time=',timeel(i),' [sec]'
      tsum = tsum+timeel(i)
      print*
      if(total_area.eq.vf*hs*vs/100) goto 21
      goto 10              
c---------------------------------------------------------
 21   np=i
      call outPS (cx1,cy1,cx2,cy2,np,xc,yc,a,1/boa,b,theta,hs,vs)
c----------------------------------------------------------
      time = dtime(tarr)
      write (*,*)
      write (*,*) 'CPU time [sec] =', tsum+tarr(1)
      write (*,*) 'tsum [sec] =', tsum

      return
      end
      
c============================================================
      SUBROUTINE chkbndc (xb1,yb1,xb2,yb2,xc,yc,r,flag)
c-------------------------------------------------------------
c Purpose:
c   Returns flag = F if inclusion contour (circle) intersects 
c   boundary (given with XB1,YB1 - left lower corner &
c   XB2,YB2 - right upper corner)
c   Inclusion is defined with its centroid XC,YC and radius R
c   pts(X1,Y1) and (X2,Y2)
c
c Req'd routines:  dist
c
c Latest revision: 5/21/95
c-------------------------------------------------------------             
      IMPLICIT REAL *8(A-H,O-Z)
 
      logical   flag

      flag = .true.   ! asume no intersect. with bound.

      call dist (xc,yc,xb1,yb1,xb2,yb1,ds) ! 1-2 bnd.ln.
      if (ds.le.r) then
        flag  = .false.
        goto 1  ! intersect.
      endif
  
      call dist (xc,yc,xb2,yb1,xb2,yb2,ds) ! 2-3 bnd.ln.
      if (ds.le.r) then
        flag  = .false.
        goto 1  ! intersect.
      endif

      call dist (xc,yc,xb2,yb2,xb1,yb2,ds) ! 3-4 bnd.ln.
      if (ds.le.r) then
        flag  = .false.
        goto 1  ! intersect.
      endif

      call dist (xc,yc,xb1,yb2,xb1,yb1,ds) ! 4-1 bnd.ln.
      if (ds.le.r) then
        flag  = .false.
        goto 1  ! intersect.
      endif

 1    RETURN 
      END

c============================================================
      SUBROUTINE chkbnde (xb1,yb1,xb2,yb2,xc,yc,a,b,th,flag)
c-------------------------------------------------------------
c Purpose:
c   Returns flag = F if elipse contour intersects 
c   boundary (given with XB1,YB1 - left lower corner &
c   XB2,YB2 - right upper corner)
c   Inclusion is defined with its centroid XC,YC and radiuses
c   a,b and orientation angle theta [deg]
c
c Req'd routines:  numint
c
c Latest revision: 6/20/95
c----------------------------------------------------------
      IMPLICIT REAL *8(A-H,O-Z)
      DIMENSION X(5),Y(5)
 
      logical   flag
      common conv,nran

      flag = .true.   ! asume no intersect. with bound.
      
      X(1)=xb1
      X(2)=xb2
      X(3)=xb2
      X(4)=xb1

      Y(1)=yb1
      Y(2)=yb1
      Y(3)=yb2
      Y(4)=yb2
      sarea=abs(xb2-xb1)*abs(yb2-yb1)
c
c     check with center point
c      
      X(5)=XC
      Y(5)=YC
      tarea=area5(x,y)
      if(tarea-sarea.GT.1.E-5) then
         flag=T
         return
      endif

      np=36
      pi=4.*atan(1.)
      thcr=th*pi/180.
      do i=1,np
         angle=i*2.*pi/np
         xi=a*dsin(angle)
         yi=b*dcos(angle)
         xr=xi*dcos(thcr)-yi*dsin(thcr)
         yr=xi*dsin(thcr)+yi*dcos(thcr)
         xa=xr+xc
         ya=yr+yc         
         X(5)=xa
         Y(5)=ya
         tarea=area5(x,y)
         if(tarea-sarea.GT.1.E-5) then
            flag=T
            return
         endif
      enddo
      return
      end

c============================================================
      subroutine gencir (xc0,yc0,r,rvar,theta,xc,yc,hs,vs)
c------------------------------------------------
c input: x/yc0 - init.circle centroid location
c        r     - circle radius
c        rvar  - radial movement factor for circle
c               centroid (0,1)
c output: x/yc - new circle location
c
c Req'd routines:  rano
c------------------------------------------------
c      implicit real*8 (a-h,o-z)
c
      real(kind=8) conv,rvar,xc0,yc0,r,hs,vs
      real(kind=8) pihlf,th1,th2,theta,cdist,xc,yc
      real ran
      integer i
      dimension timearray(3)
      common  conv,nran

      pihlf = 2.*atan(1.)

c randomly determine C variation:
      th1 = -pihlf
      th2 = pihlf
      theta = ran(nran)*(th2-th1)*conv ! [deg]
      cdist = ran(nran)*rvar
      xc = xc0+cdist*cos(theta)
      yc = yc0+cdist*sin(theta)

      return
      end

c============================================================
      subroutine genelip (xc0,yc0,ra,rb,rvar,th,xc,yc,hs,vs)
c------------------------------------------------
c input: x/yc0 - init.elip. centroid location
c        ra/b  - major/minor el. semiaxes
c        rvar  - radial movement factor for circle
c               centroid (0,1)
c output: x/yc  - new elip. location
c         th    - major semax's orientation [deg]
c
c Req'd routines:  rano
c------------------------------------------------
      implicit real*8 (a-h,o-z)

      real(kind=8) conv,rvar,xc0,yc0,ra,rb,hs,vs
      real(kind=8) pihlf,th1,th2,th,cdist,xc,yc,theta
      integer nran
      real ran
      common  conv,nran
      common   /eangle/ start_angle,end_angle

      pihlf = 2.*atan(1.)

c randomly determine C variation:
      th1 = -pihlf
      th2 = pihlf
      theta = ran(nran)*2.*(th2-th1)
      cdist = ran(nran)*rvar
      xc = xc0+cdist*cos(theta)
      yc = yc0+cdist*sin(theta)
c
c randomly determine ellipse orientation
C      th = rano(0)*(th2-th1)*180./pi   ! [deg]
      th = (start_angle+ran(nran)*(end_angle-start_angle))*conv      ! [deg]

      return
      end

c============================================================
      subroutine chkolcr (xcc,ycc,xcp,ycp,rc,rp,flag)
c----------------------------------------------------------
c Checks if a given circle (Curr.) intersects previously 
c generated circle (Prev)
c If an intersection occurs, output is FLAG=T
c
c Req'd routines: none
c----------------------------------------------------------
      implicit real*8 (A-H,O-Z)
 
      logical   flag,T,F

      T = .true.
      F = .false.
      dismin = rc+rp  ! min.allowable centr.dist.

      flag = F   ! assume no intersection
c
c First check if new center is within previously generated
c circle
      dx = xcc-xcp
      dy = ycc-ycp
      ds = sqrt(dx*dx+dy*dy)
c overlaping check:
      if (ds.le.dismin) then
        flag = T    ! intersection exists
      endif

      return
      end

c============================================================
      subroutine chkolep (xcc,ycc,ac,bc,thc,xcp,ycp,ap,bp,
     *                    thp,flag)
c----------------------------------------------------------
c Checks if a given ellipse (Curr.) intersects previously 
c generated ellipse (Prev)
c If an intersection occurs, output is FLAG=T
c
c Req'd routines: elcut
c---------------------------------------------------------
      implicit real*8 (A-H,O-Z)
 
      logical   flag,T,F

      T = .true.
      F = .false.

      flag = F     !  assume no intersection
      
      dist=dsqrt((xcc-xcp)**2+(ycc-ycp)**2)
      if(dist.lt.(bc+bp)) then
         flag = T
         return
      endif

      np=72
      pi=4.*atan(1.)
      thcr=thc*pi/180.
      thpr=thp*pi/180.
      do i=1,np
         angle=i*2.*pi/np
         xi=ac*dsin(angle)
         yi=bc*dcos(angle)
         xr=xi*dcos(thcr)-yi*dsin(thcr)
         yr=xi*dsin(thcr)+yi*dcos(thcr)
         xr=xr+xcc
         yr=yr+ycc
         xa=xr-xcp
         ya=yr-ycp         
         x=xa*dcos(thpr)+ya*dsin(thpr)
         y=-xa*dsin(thpr)+ya*dcos(thpr)         
         elp=(x/ap)**2+(y/bp)**2
         if(elp.le.1.0) then
            flag = T
            return
         endif            
      enddo
      return
      end

c============================================================
      subroutine outPS (cx1,cy1,cx2,cy2,np,xc,yc,a,boa,b,theta,hs,vs)
c---------------------------------------------------------
c Writes output data & PS file
c
c---------------------------------------------------------
      implicit real*8 (A-H,O-Z)
      parameter (k1=500)      

      integer  psf,bndx(4),bndy(4),fontsize,
     *         cols,rows,cc,rc,choice

      real*8   xc(k1),yc(k1),a(k1),boa(k1),b(k1),theta(k1)

      character*24   namein,nameps
      data     psf/101/
c
c write output file:
c
 21   print *
C--------------------------
      namein='random.dat'
      nameps='random.ps'
C--------------------------
      open (102, file=namein, status='unknown')

      write(102,*) 'Number of region`s internal openings'
      niop = 0
      write(102,*) niop

      write(102,*) 'Number of region boundary nodes'
      nbnd = 4
      write(102,*) nbnd

      if (choice.eq.2) goto 96
      write(102,*) 'Bound.node cords [in] (COUNTERCLOCKWISE):'
 96   bndx(1) = cx1
      bndy(1) = cy1
      bndx(2) = cx2
      bndy(2) = cy1
      bndx(3) = cx2
      bndy(3) = cy2
      bndx(4) = cx1
      bndy(4) = cy2
      do 3  j = 1,4
        write(102,*) bndx(j),bndy(j)
 3    continue

      fsx = cx1
      fsy = cy1

      write(102,*) 'Horiz.size of region new frame [in]:'
      write(102,*) hs

      write(102,*) 'Horiz.size of region new frame [in]:'
      write(102,*) vs

      write(102,*) 'Frame shift in the horiz. dir. [in]:'
      write(102,*) fsx

      write(102,*) 'Frame shift in the vert. dir. [in]:'
      write(102,*) fsy

 97   write(102,*) 'Number of extracted ellipses/circles:'
      write(102,'(i4)') np

      write(102,991)
 991  format(1x,'num',2x,'a(inch)',2x,'b(inch)',1x,
     *       'xc(inch)',1x,'yc(inch)',1x,'theta(dgr)',
     *       1x,'temp1',2x,'temp2')
      write(102,992)
 992  format(67('-'))

 98   do 20  i = 1,np

         if (choice.eq.2) then
            write(102,9931) xc(i),yc(i),a(i),1.,0.
 9931       format(f7.4,2x,f7.4,3x,f7.4,3x,f3.1,2x,f3.1)
            goto 20
         endif

         xc(i)=xc(i)+fsx
         yc(i)=yc(i)+fsy

         write(102,993) i,a(i),b(i),xc(i),yc(i),
     *        theta(i),0.,0.
 993     format(i4,3x,f7.4,2x,f7.4,3x,f7.4,2x,f7.4,3x,f6.1,
     *        3x,f3.1,2x,f3.1)        
 20   continue   

      close (102)
c
c---------------------------------------
c write PS picture

      scale = 8./hs ! new horiz. size will be 8 in.         
      if (vs*scale .lt. 10.) then
        hs  = 8.
        vs  = vs*scale
        fsx = 0.25
        fsy = 0.5*(11.-vs)
      else
c
c new vert. size will be 10 inches
c
        scale = 10./vs             
        hs    = scale*hs
        vs    = 10.
        fsx   = 0.5*(8.-hs)
        fsy   = 0.5
      endif

      open (psf, file=nameps, status='unknown')

      fontsize = 18
c
c Write PostScript file header
c
      cols = hs*150                ! there is 150 dpi
      rows = vs*150         

      write (psf,'(2a)') '%!'
      write (psf,801) fsx*72,fsy*72,' translate'
 801  format(1x,f7.1,1x,f7.1,1x,a10)
      write (psf,802) hs*72/cols,vs*72/rows,' scale'
 802  format(1x,f6.4,1x,f6.4,1x,a6)
      write (psf,*) ' ' 
      write (psf,*) '/Times-Roman findfont'
      write (psf,*) fontsize,' scalefont'
      write (psf,*) 'setfont'
c
c Draw the page frame (scaled to fit)
c
      write (psf,*) 'newpath'
      write (psf,*) '0 0 moveto'
      write (psf,*) cols,' 0 lineto'
      write (psf,*) cols,rows,' lineto'
      write (psf,*) '0 ',rows,' lineto'
      write (psf,*) 'closepath'
      write (psf,*) 'stroke'
      write (psf,*) ' '
c
c Draw particulates
c
      do 30  i = 1,np

        cc = xc(i)*150*scale
        rc = yc(i)*150*scale

        write (psf,*) '%% Elipse No.',i
        write (psf,*) 'gsave'
        write (psf,*) cc,rc,' translate'
        write (psf,*) -fontsize,-fontsize,' moveto'
        write (psf,800) '(',i,') show'
 800    format(1x,a1,i3,a6)
        write (psf,810) theta(i),' rotate'
 810    format(1x,f8.3,1x,a7)
        write (psf,820) '1. ',boa(i),' scale'
 820    format(1x,a3,1x,f6.4,1x,a6)
        write (psf,*) 'newpath'                  
        write (psf,830) '0 0 ',a(i)*150*scale,' 0 360 arc'
 830    format(1x,a4,1x,f6.2,1x,a10)
        write (psf,840) '0 0 ',1.5,' 0 360 arc'  ! ellips.center
 840    format(1x,a4,1x,f3.1,1x,a10)
        write (psf,*) 'stroke'
        write (psf,*) 'grestore'
        write (psf,*) ' '

 30   continue  

      close (psf)

      return
      end

C ====================================================================

      SUBROUTINE DIST (X,Y,X1,Y1,X2,Y2,DS)

c   Purpose:
c   Returns distance 'DS' of a pt. (X,Y) from a line joining
c   pts(X1,Y1) and (X2,Y2)
            
       IMPLICIT REAL *8(A-H,O-Z)
      
       A  = X2-X1
       D  = Y2-Y1
       SL = DSQRT(A*A+D*D)
       B  = D/SL
       E  = -(A/SL)
       C  = X-X1
       F  = Y-Y1

       DET= A*E-B*D
       S  = (E*C-B*F)/DET

       DS = ABS((A*F-C*D)/DET)

       RETURN 
       END

      FUNCTION AREA5(X,Y)
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION X(5),Y(5)
      AREA5=0.0
      AREA5=AREA5+
     &     ABS(X(2)*Y(5)-Y(2)*X(5)-X(1)*(Y(5)-Y(2))+Y(1)*(X(5)-X(2)))
      AREA5=AREA5+
     &     ABS(X(3)*Y(5)-Y(3)*X(5)-X(2)*(Y(5)-Y(3))+Y(2)*(X(5)-X(3)))
      AREA5=AREA5+
     &     ABS(X(4)*Y(5)-Y(4)*X(5)-X(3)*(Y(5)-Y(4))+Y(3)*(X(5)-X(4)))
      AREA5=AREA5+
     &     ABS(X(1)*Y(5)-Y(1)*X(5)-X(4)*(Y(5)-Y(1))+Y(4)*(X(5)-X(1)))
      AREA5=AREA5/2.0
      RETURN
      END

      SUBROUTINE EQ_ARC(N,A,B,ANGN,XC,YC,X,Y)
      IMPLICIT DOUBLE PRECISION (A-H,O-Z)
      DIMENSION X(1),Y(1)
      SLP=1.E0/DSQRT(2.D0)
      IF(MOD(N,4).NE.0) THEN
         WRITE(*,*) 'MOD(N,4).NE.0, INPUT N AGAIN'
         STOP
      ENDIF
      ANGLE=ANGN*DATAN(1.D0)/4.5D1
      CSM=DCOS(ANGLE)
      SNM=DSIN(ANGLE)
      TWOPI=8.0*DATAN(1.D0)
      
      X(1)=A
      Y(1)=0.D0
      X(N/4+1)=0.D0
      Y(N/4+1)=B
      X(N/4*2+1)=-A
      Y(N/4*2+1)=0.D0
      X(N/4*3+1)=0.D0
      Y(N/4*3+1)=-B
      DO I=2,N/4
         SLOPE=B/(N/4)*(I-1)/(A/(N/4)*(N/4+1-I))
         X(I)=A*B/(B*B+A*A*SLOPE*SLOPE)**0.5D0
         Y(I)=SLOPE*X(I)
      ENDDO
      DO I=2,N/4
         X(N/4+I)=-X(N/4-I+2)
         Y(N/4+I)=Y(N/4-I+2)
      ENDDO
      DO I=2,N/4
         X(N/4*2+I)=-X(I)
         Y(N/4*2+I)=-Y(I)
      ENDDO
      DO I=2,N/4
         X(N/4*3+I)=X(N/4-I+2)
         Y(N/4*3+I)=-Y(N/4-I+2)
      ENDDO
      DO I=1,N
         XBEG=X(I)*CSM-Y(I)*SNM+XC
         YBEG=Y(I)*CSM+X(I)*SNM+YC
         X(I)=XBEG
         Y(I)=YBEG
      ENDDO
      END
