#include <malloc.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <mpi.h>

#define MAXTAIL 100
#define MAXLEN 400
#define NBM 210
#define NUMIJK 200000
#define modula(a,b) ((a)-((a)/(b))*(b))
#define min(a,b) (if (a>b) b else a)

int ititle,ititle1;
int ijk_locg[NUMIJK];

int nel_vcf_map[MAXLEN];
int izero_c(int *a, int b);
int dzero_c(double *a, int b);

int inttostring(char *s, int i);
void nrerror(error_text)

char error_text[];
{
	void exit();

	fprintf(stderr,"Numerical Recipes run-time error...\n");
	fprintf(stderr,"%s\n",error_text);
	fprintf(stderr,"...now exiting to system...\n");
	exit(1);
}



float *vector(nl,nh)
int nl,nh;
{
	float *v;

	v=(float *)malloc((unsigned) (nh-nl+1)*sizeof(float));
	if (!v) nrerror("allocation failure in vector()");
	return v-nl;
}

int *ivector(nl,nh)
int nl,nh;
{
	int *v;

	v=(int *)malloc((unsigned) (nh-nl+1)*sizeof(int));
	if (!v) nrerror("allocation failure in ivector()");
	return v-nl;
}

double *dvector(nl,nh)
int nl,nh;
{
	double *v;

	v=(double *)malloc((unsigned) (nh-nl+1)*sizeof(double));
	if (!v) nrerror("allocation failure in dvector()");
	return v-nl;
}



float **matrix(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
	int i;
	float **m;

	m=(float **) malloc((unsigned) (nrh-nrl+1)*sizeof(float*));
	if (!m) nrerror("allocation failure 1 in matrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(float *) malloc((unsigned) (nch-ncl+1)*sizeof(float));
		if (!m[i]) nrerror("allocation failure 2 in matrix()");
		m[i] -= ncl;
	}
	return m;
}

double **dmatrix(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
	int i;
	double **m;

	m=(double **) malloc((unsigned) (nrh-nrl+1)*sizeof(double*));
	if (!m) nrerror("allocation failure 1 in dmatrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(double *) malloc((unsigned) (nch-ncl+1)*sizeof(double));
		if (!m[i]) nrerror("allocation failure 2 in dmatrix()");
		m[i] -= ncl;
	}
	return m;
}

int **imatrix(nrl,nrh,ncl,nch)
int nrl,nrh,ncl,nch;
{
	int i,**m;

	m=(int **)malloc((unsigned) (nrh-nrl+1)*sizeof(int*));
	if (!m) nrerror("allocation failure 1 in imatrix()");
	m -= nrl;

	for(i=nrl;i<=nrh;i++) {
		m[i]=(int *)malloc((unsigned) (nch-ncl+1)*sizeof(int));
		if (!m[i]) nrerror("allocation failure 2 in imatrix()");
		m[i] -= ncl;
	}
	return m;
}



float **submatrix(a,oldrl,oldrh,oldcl,oldch,newrl,newcl)
float **a;
int oldrl,oldrh,oldcl,oldch,newrl,newcl;
{
	int i,j;
	float **m;

	m=(float **) malloc((unsigned) (oldrh-oldrl+1)*sizeof(float*));
	if (!m) nrerror("allocation failure in submatrix()");
	m -= newrl;

	for(i=oldrl,j=newrl;i<=oldrh;i++,j++) m[j]=a[i]+oldcl-newcl;

	return m;
}



void free_vector(v,nl,nh)
float *v;
int nl,nh;
{
	free((char*) (v+nl));
}

void free_ivector(v,nl,nh)
int *v,nl,nh;
{
	free((char*) (v+nl));
}

void free_dvector(v,nl,nh)
double *v;
int nl,nh;
{
	free((char*) (v+nl));
}



void free_matrix(m,nrl,nrh,ncl,nch)
float **m;
int nrl,nrh,ncl,nch;
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}

void free_dmatrix(m,nrl,nrh,ncl,nch)
double **m;
int nrl,nrh,ncl,nch;
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}

void free_imatrix(m,nrl,nrh,ncl,nch)
int **m;
int nrl,nrh,ncl,nch;
{
	int i;

	for(i=nrh;i>=nrl;i--) free((char*) (m[i]+ncl));
	free((char*) (m+nrl));
}



void free_submatrix(b,nrl,nrh,ncl,nch)
float **b;
int nrl,nrh,ncl,nch;
{
	free((char*) (b+nrl));
}



float **convert_matrix(a,nrl,nrh,ncl,nch)
float *a;
int nrl,nrh,ncl,nch;
{
	int i,j,nrow,ncol;
	float **m;

	nrow=nrh-nrl+1;
	ncol=nch-ncl+1;
	m = (float **) malloc((unsigned) (nrow)*sizeof(float*));
	if (!m) nrerror("allocation failure in convert_matrix()");
	m -= nrl;
	for(i=0,j=nrl;i<=nrow-1;i++,j++) m[j]=a+ncol*i-ncl;
	return m;
}



void free_convert_matrix(b,nrl,nrh,ncl,nch)
float **b;
int nrl,nrh,ncl,nch;
{
	free((char*) (b+nrl));
}

int title_transfer_(char *s,int *nlm, char *title, char *title1)
{
  int i;
  int idosoft[20000];
  FILE *input_data_files;
  int MYMPI,NUMMPI;
  char stmp[MAXLEN];
  MPI_Comm_rank(MPI_COMM_WORLD,&MYMPI);
  MPI_Comm_size(MPI_COMM_WORLD,&NUMMPI);

  ititle=0;
  while (s[ititle]!=' '&&s[ititle]!=EOF&&s[ititle]!='\0'&&s[ititle]!='\n')
    {
      title[ititle]=s[ititle];
      title1[ititle]=s[ititle];
      ititle++;
    }
  title[ititle]='\0';
  title1[ititle]='\0';
  i=izero_c(nel_vcf_map,*nlm);
  i=izero_c(idosoft,*(nlm+1));
  title1=title;
  strcat(title1,"_numsele_");
  i=inttostring(stmp,0);
  strcat(title1,stmp);
  if(MYMPI==0) {
    input_data_files=fopen(title1,"w");
    fwrite(idosoft,sizeof(int),*(nlm),input_data_files);
    i=fclose(input_data_files);
  }
  MPI_Barrier(MPI_COMM_WORLD);
  title[ititle]='\0';
  return ititle;
}
int numsele_transfer_(int *nlm, char *title, char *title1)
{
  int i;
  int idosoft[20000];
  FILE *input_data_files;
  char stmp[MAXLEN];
  int MYMPI,NUMMPI;
  MPI_Comm_rank(MPI_COMM_WORLD,&MYMPI);
  MPI_Comm_size(MPI_COMM_WORLD,&NUMMPI);
  title1=title;
  strcat(title1,"_numsele_");
  i=inttostring(stmp,(*nlm)-1);
  strcat(title1,stmp);
  if(MYMPI==0) {
    input_data_files=fopen(title1,"r");
    fread(idosoft,sizeof(int),*(nlm+1),input_data_files);
    i=fclose(input_data_files);
  }
  if(NUMMPI>1) {
    MPI_Bcast(&idosoft[0],*(nlm+1),MPI_INT,0,MPI_COMM_WORLD);
  }
  title[ititle]='\0';
  title1=title;
  strcat(title1,"_numsele_");
  i=inttostring(stmp,(*nlm));
  strcat(title1,stmp);
  if(MYMPI==0){ 
    input_data_files=fopen(title1,"w");
    fwrite(idosoft,sizeof(int),*(nlm+1),input_data_files);
    i=fclose(input_data_files);
  }
  MPI_Barrier(MPI_COMM_WORLD);
  title[ititle]='\0';
  return i;
}

int refine(int nel,int *number,int *nel_side_flag)
{
  int i,k,l=1;number[nel]=1;
  for (k=0;k<4;k++)
    if ((i=nel_side_flag[nel*4+k])>0&&!number[i-1]) 
      l+=refine(i-1,number,nel_side_flag);
  return l;
}

int attatch(char *s,char *t)
{
  int i=0;
  while (*t!=EOF&&*t!='\0'&&*t!='\n')
    *s++=*t++,i++;
  *s='\0';
  return i;
}
int inttostring(char *s,int i)
{
  int j=abs(i),k=1,l=0;
  if (!i) 
    {
      s[0]='\0';
      return ("0");
    }
  while (j>9)
    {
      k++;
      j=j/10;
    }
  if (i<0) l=1;
  if (k+l>50) 
    {
      printf ("string too long??\n");
      scanf ("\n");
    }
  s[k+l]='\0',s[0]='-',j=abs(i);
  while (k)
    {
      s[(--k)+l]=48+(j-(j/10)*10);
      j=j/10;
    }
  return i;
}
int iexchn_c(int *a, int *b, int c)
{
  int i;
  for (i=0;i<c;i++)
    b[i]=a[i];
  return (1);
}
int exchn_c(double *a, double *b, int c)
{
  int i;
  for (i=0;i<c;i++)
    b[i]=a[i];
  return (1);
}
int izero_c(int *a, int b)
{
  int i;
  for (i=0;i<b;a[i++]=0);
  return (1);
}
int dzero_c(double *a, int b)
{
  int i;
  for (i=0;i<b;a[i++]=0.);
  return (1);
}

